package com.sillycycle.bagleyd.dino;

/*
 * @(#)Dino.java
 *
 * Copyright 1994 - 2023  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Random;
import javax.swing.JFrame;

/**
 * The <code>Dino</code> class extends the
 * <code>DinoCanvas</code> for handling the responses to the user
 * for the <code>DinoFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/puzzles.html
 */

/*
The Simple Solutions to Cubic Puzzles by James G. Nourse, Bantam Books,
New York, 1981. pp 8-15.

Mastering the Magic Pyramid by Tom Werneck, Evans Brothers Limited, London,
1981. pp 109-111.

Beyond Rubik's Cube: spheres, pyramids, dodecahedrons and God knows what
else by Douglas R. Hofstadter, Scientific American, July 1982, pp 16-31.

Magic Cubes 1996 Catalog of Dr. Christoph Bandelow.
*/

public class Dino extends Canvas implements MouseWheelListener,
		MouseListener, MouseMotionListener, KeyListener {
	private static final long serialVersionUID = 42L;
	public static final String NL = System.getProperty("line.separator");
	static final int MAX_ITERATIONS = 100;

	static final int AMBIGUOUS = -3;
	static final int IGNORE = -1;
	static final int TR = 0;
	static final int BR = 1;
	static final int BL = 2;
	static final int TL = 3;
	static final int STRT = 4;
	static final int CW = 5;
	static final int HALF = 6;
	static final int CCW = 7;
	static final int TOP = 8;
	static final int RIGHT = 9;
	static final int BOTTOM = 10;
	static final int LEFT = 11;
	static final int MAX_FACES = 6;
	static final int MAX_ORIENT = 4;
	static final int MAX_ROTATE = 3;
	static final int CORNER = 0;
	static final int MIDDLE = 1;
	static final int EDGE = 2;
	static final int FACE = 3;

	static final int NORMAL = 1;
	static final int DOUBLE = 2;
	static final int INSTANT = 3;
	static final String DATAFILE = "dino.dat";
	static final String SYMBOL = ":";

	Color       foreground, background;
	Color       borderColor;
	Color       currentForeground;
	Color[]     faceColor = new Color[MAX_FACES];
	int[]       directionOfPosition = null;
	int         currentDirection = -1, currentStyle = -1;
	int         currentFace = -1, nextFace = -1;
	int         currentPosition = -1, nextPosition = -1;
	boolean     nextShift = false, nextAlt = false;
	boolean     currentControl = false, nextControl = false;
	int         rowPosition;
	boolean     started, cheat;
	boolean     mono, reverse, orientChoice, practiceChoice;
	int         delay;
	int         dim;
	int         lastX, lastY;
	int         period;

	JFrame      frame = null;
	boolean     mouseDown = false;
	Random      generator;
	transient DinoStack   undo = null, redo = null;
	transient DinoSolve   solve = null;
	Dino2D      dino2D = null;
	Dino3D      dino3D = null;
	String      stringSave = null;
	String[]    token = null;
	String      faceNames = null;

	static final int[][][] facesToDirection/*[MAX_FACES][MAX_FACES][MAX_ORIENT]*/ =
	{
		{
			{-3, -3, -3, -3},
			{2, -1, 3, -3},
			{-1, 2, -3, 1},
			{1, -3, 0, -1},
			{-1, -1, -1, -1},
			{-1, 3, -3, 0}
		},
		{
			{-3, 3, -1, 0},
			{-3, -3, -3, -3},
			{1, -3, 0, -1},
			{-1, -1, -1, -1},
			{-1, 2, -3, 1},
			{2, -1, 3, -3}
		},
		{
			{-3, 3, -1, 0},
			{2, -1, 3, -3},
			{-3, -3, -3, -3},
			{1, -3, 0, -1},
			{-1, 2, -3, 1},
			{-1, -1, -1, -1}
		},
		{
			{-3, 3, -1, 0},
			{-1, -1, -1, -1},
			{2, -1, 3, -3},
			{-3, -3, -3, -3},
			{-1, 2, -3, 1},
			{1, -3, 0, -1}
		},
		{
			{-1, -1, -1, -1},
			{2, -1, 3, -3},
			{-3, 3, -1, 0},
			{1, -3, 0, -1},
			{-3, -3, -3, -3},
			{-1, 2, -3, 1}
		},
		{
			{-1, 2, -3, 1},
			{2, -1, 3, -3},
			{-1, -1, -1, -1},
			{1, -3, 0, -1},
			{-3, 3, -1, 0},
			{-3, -3, -3, -3}
		}
	};
	transient DinoLoc[][][] slideCorner = new
		DinoLoc[MAX_FACES][MAX_ORIENT][MAX_ORIENT / 2];
	static final int[] oppFace/*[MAX_FACES]*/ = {4, 3, 5, 1, 0, 2};
	transient DinoCornerLoc[][] oppCorner = new DinoCornerLoc[MAX_FACES][MAX_ORIENT];
	transient DinoCornerLoc[][] slideNextFace = new
		DinoCornerLoc[MAX_FACES][MAX_ORIENT];
	static final int[][][] faceToRotate2/*[MAX_FACES][MAX_ORIENT][2]*/ = {
		{{3, 5}, {2, 3}, {1, 2}, {1, 5}},
		{{0, 2}, {2, 4}, {4, 5}, {0, 5}},
		{{3, 0}, {4, 3}, {1, 4}, {0, 1}},
		{{0, 5}, {4, 5}, {2, 4}, {0, 2}},
		{{2, 3}, {3, 5}, {1, 5}, {1, 2}},
		{{4, 3}, {3, 0}, {0, 1}, {1, 4}}
	};

	static final int[][] faceToRotate/*[MAX_FACES][MAX_ORIENT]*/ = {
		{3, 2, 1, 5},
		{2, 4, 5, 0},
		{3, 4, 1, 0},
		{5, 4, 2, 0},
		{3, 5, 1, 2},
		{3, 0, 1, 4}
	};
	transient DinoCornerLoc[] rotateToRow = new DinoCornerLoc[MAX_FACES];
	transient DinoCornerLoc[][] cubeLoc = null;
	static transient DinoCornerLoc[][] startLoc = null;
	transient DinoCornerLoc[] faceLoc = null;
	transient DinoCornerLoc[][] rowLoc = null;
	transient DinoCornerLoc[][] spindleLoc = null;

	public Dino(JFrame frame, int dim,
			int period, boolean orient, boolean practice,
			boolean mono, boolean reverse, String names,
			Color c0, Color c1, Color c2,
			Color c3, Color c4, Color c5,
			Color borderc, Color fg, Color bg, int delay) {
		this.frame = frame;
		this.dim = dim;
		this.period = period;
		this.orientChoice = orient;
		this.practiceChoice = practice;
		this.mono = mono;
		this.reverse = reverse;
		this.faceNames = names;
		this.faceColor[0] = c0;
		this.faceColor[1] = c1;
		this.faceColor[2] = c2;
		this.faceColor[3] = c3;
		this.faceColor[4] = c4;
		this.faceColor[5] = c5;
		this.currentForeground = this.borderColor = borderc;
		this.foreground = fg; this.background = bg;
		this.delay = delay;
		dino2D = new Dino2D(frame, this);
		dino3D = new Dino3D(frame, this);
		undo = ((DinoFrame) frame).undo;
		redo = ((DinoFrame) frame).redo;
		if (dim == 2)
			dino2D.firstPaint = true;
		else if (dim == 3)
			dino3D.firstPaint = true;
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addMouseWheelListener(this);
		this.addKeyListener(this);
		setCursor(new Cursor(Cursor.HAND_CURSOR));
	}

	@Override
	protected String paramString() {
		return super.paramString() +
		",orient=" + orientChoice +
		",practice=" + practiceChoice +
		",mono=" + mono +
		",reverse=" + reverse +
		",names=" + faceNames +
		",color0=" + faceColor[0] +
		",color1=" + faceColor[1] +
		",color2=" + faceColor[2] +
		",color3=" + faceColor[3] +
		",color4=" + faceColor[4] +
		",color5=" + faceColor[5] +
		",color6=" + faceColor[6] +
		",color7=" + faceColor[7] +
		",borderColor=" + borderColor +
		",foreground=" + foreground +
		",background=" + background +
		",delay=" + delay;
	}

	void checkPieces() {
		if (period < DinoInterface.PERIOD2 ||
				period > DinoInterface.BOTH) {
			System.out.println("Mode " + period +
				" is in error, use 2 for Period2, 3 for Period3, 4 for Both, defaulting to " +
				DinoInterface.DEFAULT_MODE);
			period = DinoInterface.DEFAULT_MODE;
		}
		if (delay < 0) {
			System.out.println(delay +
			" delay cannot be negative, taking absolute value");
			delay = -delay;
		}
	}

	int getStyle(boolean shift, boolean control, boolean alt) {
		if (period == DinoInterface.BOTH) {
			if (shift)
				return EDGE;
			if (alt)
				return MIDDLE;
			return CORNER;
		}
		if (control && shift) {
			if (period == DinoInterface.PERIOD3)
				return EDGE;
			else if (alt)
				return MIDDLE;
			return CORNER;
		} else if (period == DinoInterface.PERIOD2)
			return EDGE;
		else if (alt)
			return MIDDLE;
		return CORNER;
	}

	boolean checkStarted() {
		return started;
	}

	boolean checkSolved() {
		for (int face = 0; face < MAX_FACES; face++) {
			int tempFace = cubeLoc[face][0].face;
			int tempRotation = cubeLoc[face][0].rotation;

			for (int position = 1; position < MAX_ORIENT;
					position++) {
				if (tempFace != /* face */
						cubeLoc[face][position].face ||
						(orientChoice && tempRotation != /* STRT * MAX_ORIENT */
						cubeLoc[face][position].rotation))
					return false;
			}
		}
		return true;
	}

	void drawTriangle(int face, int position, int offsetPosition) {
		if (dim == 2)
			dino2D.drawTriangle(face, position, offsetPosition);
		else if (dim == 3)
			dino3D.drawTriangle(face, position, offsetPosition);
	}

	void drawFace(int face) {
		for (int position = 0; position < MAX_ORIENT; position++)
			drawTriangle(face, position, 0);
	}

	void drawAllPieces() {
		for (int face = 0; face < MAX_FACES; face++)
			drawFace(face);
		solve.init();
	}

	void moveNoPieces() {
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_ILLEGAL, dim);
	}

	void readDiagonal(int face, int corner, int h) {
		spindleLoc[h][0] = cubeLoc[face][corner];
		spindleLoc[h][1] = cubeLoc[face][(corner + 1) % MAX_ORIENT];
	}

	void writeDiagonal(int face, int corner, int rotate, int h) {
		spindleLoc[h][0].rotation =
			(spindleLoc[h][0].rotation + rotate) % MAX_ORIENT;
		spindleLoc[h][1].rotation =
			(spindleLoc[h][1].rotation + rotate) % MAX_ORIENT;
		cubeLoc[face][corner] = spindleLoc[h][0];
		drawTriangle(face, corner, 0);
		cubeLoc[face][(corner + 1) % MAX_ORIENT] = spindleLoc[h][1];
		drawTriangle(face, (corner + 1) % MAX_ORIENT, 0);
	}

	void rotateFace(int face, int direction) {
		/* Read Face */
		for (int side = 0; side < MAX_ORIENT; side++) {
			faceLoc[side].face = cubeLoc[face][side].face;
			faceLoc[side].rotation = cubeLoc[face][side].rotation;
		}
		/* Write Face */
		for (int side = 0; side < MAX_ORIENT; side++) {
			cubeLoc[face][side].face = (direction == CW) ?
				faceLoc[(side + MAX_ORIENT - 1) % MAX_ORIENT].face :
				faceLoc[(side + 1) % MAX_ORIENT].face;
			cubeLoc[face][side].rotation =
				(cubeLoc[face][side].rotation + direction) %
				MAX_ORIENT;
			drawTriangle(face, side, 0);
		}
	}

	void readFace(int face, int h) {
		System.arraycopy(cubeLoc[face], 0, rowLoc[h], 0, MAX_ORIENT);
	}

	void writeFace(int face, int rotate, int h) {
		int newSide;

		for (int side = 0; side < MAX_ORIENT; side++) {
			newSide = (side + rotate) % MAX_ORIENT;
			cubeLoc[face][newSide] = rowLoc[h][side];
			cubeLoc[face][newSide].rotation =
				(cubeLoc[face][newSide].rotation + rotate) %
				MAX_ORIENT;
			drawTriangle(face, newSide, 0);
		}
	}

	void moveEdges(int face, int corner, int direction) {
		int oldFace = face, corn = corner, dir = direction;
		int k, newFace, rotate, newCorner, newDirection;

		readDiagonal(oldFace, corn, 0);
		for (k = 1; k <= 2; k++) {
			newFace = oppFace[oldFace];
			/*rotate = (((oldFace == 1 || oldFace == 3) ? 1 : 3) +
				3 * dir) % MAX_ORIENT; */
			newCorner = ((((((oldFace == 1 || oldFace == 3) ? 1 : 0) +
				corn) & 1) == 1) ? (corn + 3) :
				(corn + 1)) % MAX_ORIENT;
			rotate = (newCorner - corn + MAX_ORIENT) % MAX_ORIENT;
			newDirection = (rotate + dir) % MAX_ORIENT;
			if (k != 2)
				readDiagonal(newFace, newCorner, k);
			writeDiagonal(newFace, newCorner, rotate, k - 1);
			oldFace = newFace;
			corn = newCorner;
			dir = newDirection;
		}
	}

	void moveFaces(int f, int d, int rotate) {
		int k, face, newFace, rot = rotate;

		face = faceToRotate2[f][d][0];
		readFace(face, 0);
		for (k = 1; k <= 2; k++) {
			newFace = faceToRotate2[f][d][k & 1];
			rot = MAX_ORIENT - rot;
			if (k != 2)
				readFace(newFace, k);
			writeFace(newFace, rot, k - 1);
			//face = newFace;
		}
	}

	void moveInsideCorners(int face, int corner, int direction) {
		int oldFace = face, corn = corner, oldDir = direction;
		int newFace, newCorner, newDirection, dir, k;

		readDiagonal(oldFace, corn, 0);
		for (k = 1; k <= MAX_ROTATE; k++) {
			dir = oldDir / 2;
			newFace = slideCorner[oldFace][corn][dir].face;
			newCorner = slideCorner[oldFace][corn][dir].side;
			newDirection = 2 * slideCorner[oldFace][corn][dir].dir +
				(((newCorner & 1) == 0) ? 1 : 0);
			if (k != MAX_ROTATE)
				readDiagonal(newFace, newCorner, k);
			writeDiagonal(newFace, newCorner,
				(newDirection - oldDir + MAX_ORIENT) %
				MAX_ORIENT, k - 1);
			oldFace = newFace;
			corn = newCorner;
			oldDir = newDirection;
		}
	}

	void moveOutsideCorners(int face, int corner, int direction) {
		int oldFace = face, corn = corner, oldDir = direction;
		int newFace, newCorner, newDirection, dir, k;

		readDiagonal(oldFace, corn, 0);
		for (k = 1; k <= MAX_ROTATE; k++) {
			corn = (corn + 2) % MAX_ORIENT;
			dir = oldDir / 2;
			newFace = slideCorner[oldFace][corn][dir].face;
			newCorner = (slideCorner[oldFace][corn][dir].side + 2) %
				MAX_ORIENT;
			newDirection = 2 * slideCorner[oldFace][corn][dir].dir +
				(((newCorner & 1) == 0) ? 1 : 0);
			if (k != MAX_ROTATE)
				readDiagonal(newFace, newCorner, k);
			writeDiagonal(newFace, newCorner,
				(newDirection - oldDir + MAX_ORIENT) %
				MAX_ORIENT, k - 1);
			oldFace = newFace;
			corn = newCorner;
			oldDir = newDirection;
		}
	}

	void moveFacesOrth(int face, int direction) {
		int newFace, newDirection, k, rotate;
                int oldFace = face, dir = direction;

		rotateFace(faceToRotate[oldFace][dir % MAX_ORIENT],
			CW);
		rotateFace(faceToRotate[oldFace][(dir + 2) % MAX_ORIENT],
			CCW);
		readFace(oldFace, 0);
		for (k = 1; k <= MAX_ORIENT; k++) {
			newFace = slideNextFace[oldFace][dir %
				MAX_ORIENT].face;
			rotate = slideNextFace[oldFace][dir %
				MAX_ORIENT].rotation;
			newDirection = (rotate + dir) % MAX_ORIENT;
			if (k != MAX_ORIENT)
				readFace(newFace, k);
			writeFace(newFace, rotate, k - 1);
			oldFace = newFace;
			dir = newDirection;
		}
	}

	void movePieces(int face, int position, int direction, int style) {
		int oldFace = face, dir = direction;
		int corner, newCorner;

		corner = (position - ((((position + dir) & 1) == 0) ?
			1 : 0) + MAX_ORIENT) % MAX_ORIENT;
		switch (style) {
		case CORNER:
			moveInsideCorners(oldFace, corner, dir);
			break;
		case MIDDLE:
			moveOutsideCorners(oldFace, corner, dir);
			newCorner = oppCorner[oldFace][corner].rotation;
			oldFace = oppCorner[oldFace][corner].face;
			if (((((oldFace == 1 || oldFace == 3) ? 0 : 1) + corner) &
					1) == 1)
				dir = (dir + 1) % MAX_ORIENT;
			else
				dir = (dir + 3) % MAX_ORIENT;
			corner = newCorner;
			moveOutsideCorners(oldFace, corner, dir);
			break;
		case EDGE:
			moveEdges(oldFace, corner, dir);
			if (oldFace == 0 && corner == 1 && (dir & 1) == 1)
				moveFaces(oldFace, corner, 0);
			else if (oldFace == 0 && corner == 3 && (dir & 1) == 0)
				moveFaces(oldFace, corner, 0);
			else if (oldFace == 0 && corner == 0)
				moveFaces(oldFace, corner, 0);
			else if (oldFace == 4 && corner == 1 && (dir & 1) == 0)
				moveFaces(oldFace, corner, 0);
			else if (oldFace == 4 && corner == 3 && (dir & 1) == 1)
				moveFaces(oldFace, corner, 0);
			else if (oldFace == 4 && corner == 2)
				moveFaces(oldFace, corner, 0);
			else
				moveFaces(oldFace, corner,
					((oldFace == 2 || oldFace == 5) ? CCW : HALF) %
					MAX_ORIENT);
			break;
		default: /* FACE */
			if (direction >= 2 * MAX_ORIENT) {
                        	moveFacesOrth(face, direction);
			}
			break;
		}
	}

	void moveControlCb(int face, int position, int direction, int style) {
		int newFace, newSide, newDirection, corner, newCorner;

		if (style == FACE) {
			int pos = position, dir = direction;

			newFace = face;
			if (direction == CW || direction == CCW) {
				newFace = rotateToRow[face].face;
				dir = 2 * MAX_ORIENT + ((direction == CCW) ?
					(rotateToRow[face].rotation + 2) :
					rotateToRow[face].rotation) % MAX_ORIENT;
			}
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CONTROL, dim,
				newFace, pos, dir, FACE);
			moveFacesOrth(newFace, dir);
			return;
                }
		movePieces(face, position, direction, style);
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_CONTROL, dim,
			face, position, direction, style);
		switch (style) {
		case CORNER:
			newSide = (position + 2) % MAX_ORIENT;
			movePieces(face, newSide, direction, MIDDLE);
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CONTROL, dim,
				face, newSide, direction, MIDDLE);
			corner = (position - ((((position + direction) & 1) ==
				0) ? 1 : 0) + MAX_ORIENT) % MAX_ORIENT;
			newFace = oppCorner[face][corner].face;
			newCorner = oppCorner[face][corner].rotation;
			newDirection = 2 * ((direction / 2 +
				((face == 1 || face == 3) ? 0 : 1)) & 1) +
				(((newCorner & 1) == 0) ? 1 : 0);
			newSide = newCorner;
			movePieces(newFace, newSide, newDirection, CORNER);
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CONTROL, dim,
				newFace, newSide, newDirection, CORNER);
			break;
		case MIDDLE:
			newSide = (position + 2) % MAX_ORIENT;
			movePieces(face, newSide, direction, CORNER);
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CONTROL, dim,
				face, newSide, direction, CORNER);
			corner = (position - ((((position + direction) & 1) ==
				0) ? 1 : 0) + MAX_ORIENT) % MAX_ORIENT;
			newCorner = oppCorner[face][corner].rotation;
			newFace = oppCorner[face][corner].face;
			if (newFace == 1 || newFace == 3)
				newDirection = direction;
			else
				newDirection = (direction + 2) % MAX_ORIENT;
			newDirection = 2 * (newDirection / 2) +
				(((newDirection & 1) == 0) ? 1 : 0);
			newSide = (newCorner + 2) % MAX_ORIENT;
			movePieces(newFace, newSide, newDirection, CORNER);
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CONTROL, dim,
					newFace, newSide, newDirection, CORNER);
			break;
		case EDGE:
			newSide = (position + 2) % MAX_ORIENT;
			movePieces(face, newSide, direction, EDGE);
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CONTROL, dim,
				face, newSide, direction, EDGE);
			break;
		default:
			break;
		}
	}

	void movePuzzle(int face, int position, int direction, int style,
			boolean control) {
		if (control) {
			moveControlCb(face, position, direction, style);
		} else {
			movePieces(face, position, direction, style);
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_MOVED, dim,
				face, position, direction, style);
		}
		undo.setMove(direction, style, (control) ? 1 : 0,
			face, position);
		redo.flushMoves();
		if (((DinoFrame) frame).getToggleSound())
			((DinoFrame) frame).playMoveAudio();
	}

	void movePuzzleDelay(int face, int position, int direction, int style,
			boolean control, int fast) {
		movePuzzle(face, position, direction, style, control);
		try {
			Thread.sleep(64 * delay / fast);
		} catch (InterruptedException e) {
			// ignore
		}
	}

	void selectPieces(/*int x, int y*/) {
		if (!nextControl && !practiceChoice && checkSolved())
			moveNoPieces();
		else if (nextFace >= 0) {
			int count;

			currentStyle = getStyle(nextShift, nextControl,
				nextAlt);
			if (nextFace == currentFace) {
				// next sets currentDirection
				count = checkMoveDir(currentPosition,
					nextPosition);
				/* count == 2 too annoying */
				if (count == 2)
					count = -1;
			} else {
				currentDirection = facesToDirection[currentFace][nextFace][currentPosition];
				if (currentDirection == AMBIGUOUS &&
						(currentStyle == CORNER ||
						currentStyle == MIDDLE)) {
					/* minimize ambiguity */
					count = nextFace;
					nextFace = currentFace;
					currentFace = count;
					currentPosition = nextPosition;
					currentDirection =
						facesToDirection[currentFace][nextFace][currentPosition];
					if (currentDirection >= 0)
						currentDirection = (currentDirection +
							MAX_ORIENT / 2) % MAX_ORIENT;
				}
				count = (currentDirection == AMBIGUOUS) ?
					2 : 1;
				if (currentDirection ==
						DinoInterface.ACTION_IGNORE)
					count = 0;
			}
			switch (count) {
			case 1:
				movePuzzle(currentFace, currentPosition,
					currentDirection, currentStyle,
					nextControl);
				started = true;
				if (!nextControl && checkSolved()) {
					((DinoFrame) frame).callback(
						DinoInterface.ACTION_SOLVED,
						dim);
				}
				break;
			case 2:
				((DinoFrame) frame).callback(
					DinoInterface.ACTION_AMBIGUOUS, dim);
				break;
			case 0:
				moveNoPieces();
				break;
			default:
				break;
			}
		}
	}

	int checkMoveDir(int position1, int position2) {
		if (position1 == position2)
			return 2;
		if (((position1 - position2 + MAX_ORIENT) & 1) == 0)
			return 0;
		switch (position1) {
		case 0:
			currentDirection = ((position2 == 1) ? 1 : 2);
			break;
		case 1:
			currentDirection = ((position2 == 2) ? 2 : 3);
			break;
		case 2:
			currentDirection = ((position2 == 3) ? 3 : 0);
			break;
		case 3:
			currentDirection = ((position2 == 0) ? 0 : 1);
			break;
		default:
			return 0;
		}
		return 1;
	}

	boolean narrowSelection() {
		// sets currentFace, currentPosition, currentDirection
		if (dim == 2)
			dino2D.narrowSelection();
		else if (dim == 3)
			dino3D.narrowSelection();
		return true;
	}

	boolean positionPieces(int x, int y, int direction) {
		// sets currentFace, currentPosition
		if (dim == 2) {
			currentFace = dino2D.positionToPiece(x, y);
		} else if (dim == 3) {
			currentFace = dino3D.positionToPiece(x, y);
		}
		if (currentFace < 0)
			return false;
		currentDirection = direction;
		narrowSelection(); // sets currentFace, currentPosition, currentDirection
		return true;
	}

	void movePuzzleInput(int x, int y, int direction,
			boolean shift, boolean control, boolean alt) {
		if (!practiceChoice && !control && !alt && checkSolved()) {
			moveNoPieces();
			return;
		}
		if (!positionPieces(x, y, direction)) // sets currentFace, currentPosition, currentDirection
			return;
		if (direction >= MAX_ORIENT) {
			if (control)
				currentStyle = FACE;
			else
				return;
		} else
			currentStyle = getStyle(shift, control, alt);
		movePuzzle(currentFace, currentPosition, currentDirection,
			currentStyle, control);
		if (!control && checkSolved()) {
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_SOLVED, dim);
		}
	}

	void resetPieces() {
		cubeLoc = new DinoCornerLoc[MAX_FACES][MAX_ORIENT];
		startLoc = new DinoCornerLoc[MAX_FACES][MAX_ORIENT];
		rowLoc = new DinoCornerLoc[MAX_ORIENT][MAX_ORIENT];
		spindleLoc = new DinoCornerLoc[MAX_ROTATE][2];
		faceLoc = new DinoCornerLoc[MAX_ORIENT];
		for (int face = 0; face < MAX_FACES; face++) {
			for (int position = 0; position < MAX_ORIENT;
					position++) {
				startLoc[face][position] = new DinoCornerLoc(
					face, STRT - MAX_ORIENT);
				cubeLoc[face][position] = new DinoCornerLoc(
					face, STRT - MAX_ORIENT);
			}
		}
		for (int orient = 0; orient < MAX_ORIENT; orient++) {
			for (int position = 0; position < MAX_ORIENT; position++)
				rowLoc[orient][position] = new DinoCornerLoc(0,
					0);
			faceLoc[orient] = new DinoCornerLoc(0, STRT - MAX_ORIENT);
		}
		for (int rotate = 0; rotate < MAX_ROTATE; rotate++) {
			spindleLoc[rotate][0] = new DinoCornerLoc(0, 0);
			spindleLoc[rotate][1] = new DinoCornerLoc(0, 0);
		}
		undo.flushMoves();
		redo.flushMoves();
		getStartPosition();
		currentFace = -1;
		currentPosition = -1;
		currentDirection = -1;
		started = false;
		cheat = false;
		dino2D.randomPaints = 0;
		dino3D.randomPaints = 0;
	}

	void practicePieces() {
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_PRACTICE, dim);
	}

	void randomizePieces() {
		if (practiceChoice)
			practicePieces();
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_RESET, dim);
		int big = MAX_ORIENT * 3;
		if (dim == 2)
			dino2D.randomPaints = java.lang.Math.min(big,
				MAX_ITERATIONS) + generator.nextInt(2);
		else if (dim == 3)
			dino3D.randomPaints = java.lang.Math.min(big,
				MAX_ITERATIONS) + generator.nextInt(2);
		repaint();
	}

	void randomizingPieces() {
		int position, randomDirection, style;
		int face = generator.nextInt(MAX_FACES);

		position = generator.nextInt(MAX_ORIENT);
		randomDirection = ((generator.nextInt(2) == 1) ? position + 1 :
			position + 3) % MAX_ORIENT;
		switch (period) {
		case DinoInterface.PERIOD2:
			style = EDGE;
			break;
		case DinoInterface.BOTH:
			style = generator.nextInt(3);
			break;
		default:
			style = generator.nextInt(2);

		}
		movePuzzle(face, position, randomDirection, style, false);
		if (dim == 2)
			dino2D.randomPaints--;
		else if (dim == 3)
			dino3D.randomPaints--;
		try {
			Thread.sleep(delay);
		} catch (InterruptedException e) {
			// ignore
		}
		if (dim == 2 && dino2D.randomPaints > 0) {
			repaint();
		} else if (dim == 3 && dino3D.randomPaints > 0) {
			repaint();
		} else {
			undo.flushMoves();
			redo.flushMoves();
			getStartPosition();
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_RANDOMIZE, dim);
			started = false;
			cheat = false;
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CHEAT, dim);
			currentFace = -1;
			if (checkSolved())
				((DinoFrame) frame).callback(
					DinoInterface.ACTION_SOLVED, dim);
		}
	}

	public int getDelay() {
		return delay;
	}

	public void setDelay(int delay) {
		this.delay = delay;
	}

	public boolean getCheat() {
		return cheat;
	}

	public void setCheat(boolean cheater) {
		cheat = cheater;
	}

	public void setStart(boolean start) {
		started = start;
	}

	boolean setPuzzlePeriod(int newValue) {
		if (period != newValue &&
				newValue >= DinoInterface.PERIOD2) {
			period = newValue;
			resetPieces();
			if (dim == 2)
				dino2D.resizePaint = true;
			else if (dim == 3)
				dino3D.resizePaint = true;
			repaint();
			return true;
		}
		return false;
	}

	boolean setPuzzleOrient(boolean newValue) {
		if (orientChoice != newValue) {
			orientChoice = newValue;
			resetPieces();
			if (dim == 2)
				dino2D.resizePaint = true;
			else if (dim == 3)
				dino3D.resizePaint = true;
			repaint();
			return true;
		}
		return false;
	}

	boolean setPuzzlePractice(boolean newValue) {
		if (practiceChoice != newValue) {
			practiceChoice = newValue;
			resetPieces();
			if (dim == 2)
				dino2D.resizePaint = true;
			else if (dim == 3)
				dino3D.resizePaint = true;
			repaint();
			return true;
		}
		return false;
	}

	void selectPuzzle(boolean control, int x, int y) {
		if (dim == 2)
			dino2D.selectPuzzle(control, x, y);
		else if (dim == 3)
			dino3D.selectPuzzle(control, x, y);
	}

	void releasePuzzle(boolean shift, boolean control, boolean alt,
			int x, int y) {
		if (dim == 2)
			dino2D.releasePuzzle(shift, control, alt, x, y);
		else if (dim == 3)
			dino3D.releasePuzzle(shift, control, alt, x, y);
	}

	void randomizePuzzle() {
		if (!mouseDown)
			randomizePieces();
	}

	void randomizePuzzleWithQuery() {
		if (!started)
			randomizePuzzle();
	}

	public void speedUpPuzzle() {
		delay -= 10;
		if (delay < 0)
			delay = 0;
		((DinoFrame) frame).callback(DinoInterface.ACTION_SPEED, dim);
	}

	public void slowDownPuzzle() {
		delay += 10;
		((DinoFrame) frame).callback(DinoInterface.ACTION_SPEED, dim);
	}

	void showMessage(String msg) {
		((DinoFrame) frame).showMessage(msg);
	}

	public void initializePuzzle() {
		checkPieces();
		solve = new DinoSolve(frame, this);
		resetPieces();
		generator = new Random(System.nanoTime());
		rotateToRow[0] = new DinoCornerLoc(1, LEFT);
		rotateToRow[1] = new DinoCornerLoc(0, BOTTOM);
		rotateToRow[2] = new DinoCornerLoc(0, RIGHT);
		rotateToRow[3] = new DinoCornerLoc(0, TOP);
		rotateToRow[4] = new DinoCornerLoc(1, RIGHT);
		rotateToRow[5] = new DinoCornerLoc(1, LEFT);
		slideCorner[0][0][0] = new DinoLoc(3, TR, 0);
		slideCorner[0][0][1] = new DinoLoc(5, BR, 0);
		slideCorner[0][1][0] = new DinoLoc(3, TL, 1);
		slideCorner[0][1][1] = new DinoLoc(2, TR, 0);
		slideCorner[0][2][0] = new DinoLoc(2, TL, 1);
		slideCorner[0][2][1] = new DinoLoc(1, TR, 0);
		slideCorner[0][3][0] = new DinoLoc(5, BL, 1);
		slideCorner[0][3][1] = new DinoLoc(1, TL, 1);
		slideCorner[1][0][0] = new DinoLoc(2, TL, 0);
		slideCorner[1][0][1] = new DinoLoc(0, BL, 0);
		slideCorner[1][1][0] = new DinoLoc(2, BL, 0);
		slideCorner[1][1][1] = new DinoLoc(4, TL, 0);
		slideCorner[1][2][0] = new DinoLoc(4, BL, 0);
		slideCorner[1][2][1] = new DinoLoc(5, TL, 0);
		slideCorner[1][3][0] = new DinoLoc(0, TL, 0);
		slideCorner[1][3][1] = new DinoLoc(5, BL, 0);
		slideCorner[2][0][0] = new DinoLoc(3, TL, 0);
		slideCorner[2][0][1] = new DinoLoc(0, BR, 0);
		slideCorner[2][1][0] = new DinoLoc(3, BL, 0);
		slideCorner[2][1][1] = new DinoLoc(4, TR, 0);
		slideCorner[2][2][0] = new DinoLoc(4, TL, 1);
		slideCorner[2][2][1] = new DinoLoc(1, BR, 1);
		slideCorner[2][3][0] = new DinoLoc(0, BL, 1);
		slideCorner[2][3][1] = new DinoLoc(1, TR, 1);
		slideCorner[3][0][0] = new DinoLoc(5, BR, 1);
		slideCorner[3][0][1] = new DinoLoc(0, TR, 1);
		slideCorner[3][1][0] = new DinoLoc(5, TR, 1);
		slideCorner[3][1][1] = new DinoLoc(4, BR, 1);
		slideCorner[3][2][0] = new DinoLoc(4, TR, 1);
		slideCorner[3][2][1] = new DinoLoc(2, BR, 1);
		slideCorner[3][3][0] = new DinoLoc(0, BR, 1);
		slideCorner[3][3][1] = new DinoLoc(2, TR, 1);
		slideCorner[4][0][0] = new DinoLoc(3, BL, 1);
		slideCorner[4][0][1] = new DinoLoc(2, BR, 0);
		slideCorner[4][1][0] = new DinoLoc(3, BR, 0);
		slideCorner[4][1][1] = new DinoLoc(5, TR, 0);
		slideCorner[4][2][0] = new DinoLoc(5, TL, 1);
		slideCorner[4][2][1] = new DinoLoc(1, BL, 1);
		slideCorner[4][3][0] = new DinoLoc(2, BL, 1);
		slideCorner[4][3][1] = new DinoLoc(1, BR, 0);
		slideCorner[5][0][0] = new DinoLoc(3, BR, 1);
		slideCorner[5][0][1] = new DinoLoc(4, BR, 0);
		slideCorner[5][1][0] = new DinoLoc(3, TR, 1);
		slideCorner[5][1][1] = new DinoLoc(0, TR, 0);
		slideCorner[5][2][0] = new DinoLoc(0, TL, 1);
		slideCorner[5][2][1] = new DinoLoc(1, TL, 0);
		slideCorner[5][3][0] = new DinoLoc(4, BL, 1);
		slideCorner[5][3][1] = new DinoLoc(1, BL, 0);
		oppCorner[0][0] = new DinoCornerLoc(4, 3);
		oppCorner[0][1] = new DinoCornerLoc(4, 2);
		oppCorner[0][2] = new DinoCornerLoc(4, 1);
		oppCorner[0][3] = new DinoCornerLoc(4, 0);
		oppCorner[1][0] = new DinoCornerLoc(3, 1);
		oppCorner[1][1] = new DinoCornerLoc(3, 0);
		oppCorner[1][2] = new DinoCornerLoc(3, 3);
		oppCorner[1][3] = new DinoCornerLoc(3, 2);
		oppCorner[2][0] = new DinoCornerLoc(5, 3);
		oppCorner[2][1] = new DinoCornerLoc(5, 2);
		oppCorner[2][2] = new DinoCornerLoc(5, 1);
		oppCorner[2][3] = new DinoCornerLoc(5, 0);
		oppCorner[3][0] = new DinoCornerLoc(1, 1);
		oppCorner[3][1] = new DinoCornerLoc(1, 0);
		oppCorner[3][2] = new DinoCornerLoc(1, 3);
		oppCorner[3][3] = new DinoCornerLoc(1, 2);
		oppCorner[4][0] = new DinoCornerLoc(0, 3);
		oppCorner[4][1] = new DinoCornerLoc(0, 2);
		oppCorner[4][2] = new DinoCornerLoc(0, 1);
		oppCorner[4][3] = new DinoCornerLoc(0, 0);
		oppCorner[5][0] = new DinoCornerLoc(2, 3);
		oppCorner[5][1] = new DinoCornerLoc(2, 2);
		oppCorner[5][2] = new DinoCornerLoc(2, 1);
		oppCorner[5][3] = new DinoCornerLoc(2, 0);
		slideNextFace[0][0] = new DinoCornerLoc(5, STRT);
		slideNextFace[0][1] = new DinoCornerLoc(3, CW);
		slideNextFace[0][2] = new DinoCornerLoc(2, STRT);
		slideNextFace[0][3] = new DinoCornerLoc(1, CCW);
		slideNextFace[1][0] = new DinoCornerLoc(0, CW);
		slideNextFace[1][1] = new DinoCornerLoc(2, STRT);
		slideNextFace[1][2] = new DinoCornerLoc(4, CCW);
		slideNextFace[1][3] = new DinoCornerLoc(5, HALF);
		slideNextFace[2][0] = new DinoCornerLoc(0, STRT);
		slideNextFace[2][1] = new DinoCornerLoc(3, STRT);
		slideNextFace[2][2] = new DinoCornerLoc(4, STRT);
		slideNextFace[2][3] = new DinoCornerLoc(1, STRT);
		slideNextFace[3][0] = new DinoCornerLoc(0, CCW);
		slideNextFace[3][1] = new DinoCornerLoc(5, HALF);
		slideNextFace[3][2] = new DinoCornerLoc(4, CW);
		slideNextFace[3][3] = new DinoCornerLoc(2, STRT);
		slideNextFace[4][0] = new DinoCornerLoc(2, STRT);
		slideNextFace[4][1] = new DinoCornerLoc(3, CCW);
		slideNextFace[4][2] = new DinoCornerLoc(5, STRT);
		slideNextFace[4][3] = new DinoCornerLoc(1, CW);
		slideNextFace[5][0] = new DinoCornerLoc(4, STRT);
		slideNextFace[5][1] = new DinoCornerLoc(3, HALF);
		slideNextFace[5][2] = new DinoCornerLoc(0, STRT);
		slideNextFace[5][3] = new DinoCornerLoc(1, HALF);
	}

	void exposePuzzle() {
		if (dim == 2)
			dino2D.exposePuzzle();
		else if (dim == 3)
			dino3D.exposePuzzle();
	}

	public void getPuzzle(boolean incMoves) {
		this.stringSave = ((DinoFrame) frame).readTextArea();
		if (this.stringSave == null || this.stringSave.equals("")) {
			if (incMoves)
				showMessage("Empty string to read");
			return;
		}
		getPuzzleString("buffer", incMoves);
	}

	public void getPuzzleFile(boolean incMoves) {
		undo.flushMoves();
		redo.flushMoves();
		getStartPosition();
		try (BufferedReader br = new BufferedReader(
				new FileReader(DATAFILE))) {
			StringBuilder puzzleData = new StringBuilder();
			String lineData;

			while ((lineData = br.readLine()) != null)
				puzzleData.append(lineData).append(NL);
			this.stringSave = puzzleData.toString();
			if (this.stringSave == null || this.stringSave.length() == 0) {
				if (incMoves)
					showMessage("Empty file " + DATAFILE);
				return;
			}
			getPuzzleString(DATAFILE, incMoves);
		} catch (Exception e) {
			//e.printStackTrace();
			if (incMoves)
				showMessage("Cannot read " + DATAFILE);
		}
	}

	public void getPuzzleString(String source, boolean incMoves) {
		try {
			int newPeriod, moves;
			boolean orient, practice;
			String symbol = SYMBOL;
			String newline = NL;
			int line = 0;

			token = this.stringSave.split(newline);
			String lineData = token[line];
			if (lineData == null || lineData.length() == 0) {
				if (incMoves)
					showMessage(source +
						" nothing in buffer");
				return;
			}
			newPeriod = Integer.parseInt(lineData.substring(
				lineData.indexOf(symbol) + 2));
			if (newPeriod >= DinoInterface.PERIOD2 &&
					newPeriod <= DinoInterface.BOTH) {
				if (newPeriod != this.period) {
					int reason = newPeriod -
						DinoInterface.PERIOD2 +
						DinoInterface.ACTION_PERIOD2;

					((DinoFrame) frame).callback(reason, dim);
				}
			} else {
				if (incMoves)
					showMessage(source +
						" corrupted: period " + newPeriod +
						" should be even and between " +
						DinoInterface.PERIOD2 +
						" and " + DinoInterface.BOTH +
						" inclusive");
				return;
			}
			lineData = token[++line];
			orient = (Integer.parseInt(lineData.substring(
				lineData.indexOf(symbol) + 2)) != 0);
			if (this.orientChoice != orient)
				((DinoFrame) frame).callback(
					DinoInterface.ACTION_ORIENTIZE, dim);
			lineData = token[++line];
			practice = (Integer.parseInt(lineData.substring(
				lineData.indexOf(symbol) + 2)) != 0);
			if (this.practiceChoice != practice)
				((DinoFrame) frame).callback(
					DinoInterface.ACTION_PRACTICE, dim);
			lineData = token[++line];
			moves = Integer.parseInt(lineData.substring(
				lineData.indexOf(symbol) + 2));
			line = scanStartPosition(++line, incMoves);
			if (line < 0)
				return;
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_RESTORE, dim);
			setStartPosition();
			if (dim == 2)
				dino2D.allPiecesPaint = true;
			else if (dim == 3)
				dino3D.allPiecesPaint = true;
			repaint();
			if (!scanMoves(++line, moves, incMoves))
				return;
			System.out.println(source + ": period " + newPeriod +
				", orient " + orient +
				", practice " + practice + ", moves " + moves);
			cheat = true;
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_CHEAT, dim);
		} catch (NumberFormatException e) {
			//e.printStackTrace();
			if (incMoves)
				showMessage("Corrupt input when reading");
		} catch (Exception e) {
			//e.printStackTrace();
			if (incMoves)
				showMessage("Bad input file");
		}
	}

	public void printPuzzle(StringBuilder sb) {
		String symbol = SYMBOL;
		String space = " ";
		String newline = NL;

		sb.append("period").append(symbol).append(space)
						.append(period).append(newline);
		sb.append("orient").append(symbol).append(space)
						.append((orientChoice) ? 1 : 0).append(newline);
		sb.append("practice").append(symbol).append(space)
						.append((practiceChoice) ? 1 : 0).append(newline);
		sb.append("moves").append(symbol).append(space)
						.append(undo.numMoves()).append(newline);
		printStartPosition(sb);
		printMoves(sb);
	}

	public void writePuzzle() {
		StringBuilder sb = new StringBuilder();

		printPuzzle(sb);
		((DinoFrame) frame).writeTextArea(sb.toString());
	}

	public void writePuzzleFile() {
		StringBuilder sb = new StringBuilder();

		printPuzzle(sb);
		try (PrintWriter pr = new PrintWriter(new FileWriter(
				DATAFILE, false))) {
			pr.print(sb);
			System.out.println("Saved to " + DATAFILE);
		} catch (Exception e) {
			//e.printStackTrace();
			showMessage("Cannot write to " + DATAFILE);
		}
	}

	void undoPuzzle() {
		if (mouseDown || dino2D.stackPaint || dino3D.stackPaint ||
				dino2D.randomPaints != 0 || dino3D.randomPaints != 0 ||
				!undo.madeMoves())
			return;
		undo.getMove();
		currentDirection = undo.readMoveDirection();
		currentStyle = undo.readMoveStyle();
		currentControl = (undo.readMoveControl() != 0);
		currentFace = undo.readMoveFace();
		currentPosition = undo.readMovePosition();
		redo.setMove(currentDirection, currentStyle, (currentControl) ?
			1 : 0, currentFace, currentPosition);
		// move in opposite direction
		currentDirection = (currentDirection < MAX_ORIENT) ?
			(currentDirection + MAX_ORIENT / 2) %
			MAX_ORIENT : 5 * MAX_ORIENT - currentDirection;
		if (currentControl) {
			if (dim == 2)
				dino2D.stackPaint = true;
			else if (dim == 3)
				dino3D.stackPaint = true;
			repaint();
		} else /* if (checkmovePiecesDir(currentDirection)) */ {
			if (dim == 2)
				dino2D.stackPaint = true;
			else if (dim == 3)
				dino3D.stackPaint = true;
			repaint();
			started = true;
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_UNDO, dim,
				currentFace, currentPosition, currentDirection,
				currentStyle);
		} /* else currentDirection = currentFace = -1; */
	}

	void redoPuzzle() {
		if (mouseDown || dino2D.stackPaint || dino3D.stackPaint ||
				dino2D.randomPaints != 0 || dino3D.randomPaints != 0 ||
				!redo.madeMoves())
			return;
		redo.getMove();
		currentDirection = redo.readMoveDirection();
		currentStyle = redo.readMoveStyle();
		currentControl = (redo.readMoveControl() != 0);
		currentFace = redo.readMoveFace();
		currentPosition = redo.readMovePosition();
		undo.setMove(currentDirection, currentStyle, (currentControl) ?
			1 : 0, currentFace, currentPosition);
		if (currentControl) {
			if (dim == 2)
				dino2D.stackPaint = true;
			else if (dim == 3)
				dino3D.stackPaint = true;
			repaint();
		} else /* if (checkmovePiecesDir(currentDirection)) */ {
			if (dim == 2)
				dino2D.stackPaint = true;
			else if (dim == 3)
				dino3D.stackPaint = true;
			repaint();
			started = true;
			((DinoFrame) frame).callback(
				DinoInterface.ACTION_REDO, dim,
				currentFace, currentPosition, currentDirection,
				currentStyle);
		} /* else currentDirection = currentFace = -1; */
	}

	public void clearPuzzle() {
		if (mouseDown || dino2D.randomPaints != 0 ||
				dino3D.randomPaints != 0)
			return;
		resetPieces();
		drawAllPieces();
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_CLEAR, dim);
		if (dim == 2)
			dino2D.allPiecesPaint = true;
		else if (dim == 3)
			dino3D.allPiecesPaint = true;
		repaint();
	}

	void solvePuzzle() {
		if (mouseDown || dino2D.stackPaint || dino3D.stackPaint ||
				dino2D.randomPaints != 0 || dino3D.randomPaints != 0)
			return;
		cheat = true;
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_CHEAT, dim);
		solve.setFlag(true);
		try {
			Thread.sleep(200);
		} catch (InterruptedException e) {
			System.err.println("Interrupted");
		}
		//solve.solvePieces();
	}

	void orientizePuzzle() {
		if (mouseDown || dino2D.randomPaints != 0 ||
				dino3D.randomPaints != 0)
			return;
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_ORIENTIZE, dim);
	}

	void practicePuzzle() {
		if (mouseDown || dino2D.randomPaints != 0 ||
				dino3D.randomPaints != 0)
			return;
		practicePieces();
	}

	void practicePuzzleWithQuery() {
		if (!started)
			practicePuzzle();
	}

	static void printPositions() {
		for (int face = 0; face < MAX_FACES; face++) {
			for (int position = 0; position < MAX_ORIENT; position++) {
				int pos = startLoc[face][position].face;
				int rot = startLoc[face][position].rotation;

				if (pos < 100)
					System.out.print(" ");
				if (pos < 10 && pos >= 0)
					System.out.print(" ");
				System.out.print(pos + "," + rot);
				if (position == MAX_ORIENT - 1) {
					System.out.println();
				} else {
					System.out.print(" ");
				}
			}
			System.out.println();
		}
	}

	void printCube() {
		for (int face = 0; face < MAX_FACES; face++) {
			for (int position = 0; position < MAX_ORIENT; position++)
				System.out.print(
					cubeLoc[face][position].face + " " +
					cubeLoc[face][position].rotation + "  ");
			System.out.println();
		}
		System.out.println();
	}

	void printFace() {
		for (int position = 0; position < MAX_ORIENT; position++)
			System.out.print(
				faceLoc[position].face + " " +
				faceLoc[position].rotation + "  ");
		System.out.println();
	}


	void printRow(int orient, int size) {
		System.out.println("Row " + orient + ":");
		for (int g = 0; g < size; g++)
			System.out.print(
					rowLoc[orient][g].face + " " +
					rowLoc[orient][g].rotation +
					"  ");
		System.out.println();
	}

	void getStartPosition() {
		for (int face = 0; face < MAX_FACES; face++)
			for (int position = 0; position < MAX_ORIENT; position++) {
				startLoc[face][position].face =
					cubeLoc[face][position].face;
				startLoc[face][position].rotation =
					cubeLoc[face][position].rotation;
			}
	}

	void setStartPosition() {
		for (int face = 0; face < MAX_FACES; face++)
			for (int position = 0; position < MAX_ORIENT; position++) {
				cubeLoc[face][position].face =
					startLoc[face][position].face;
				cubeLoc[face][position].rotation =
					startLoc[face][position].rotation;
			}
		// drawAll
	}

	int scanStartPosition(int location, boolean incMoves) {
		String lineData;
		int line = location;

		try {
			String space = "\\s+";
			int group = (orientChoice) ? 2 : 1;

			++line; //read startingPosition:
			for (int face = 0; face < MAX_FACES; face++) {
				line++;
				if (line >= token.length) {
					if (incMoves)
						System.out.println("Short data in StartPosition");
					return -1;
				}
				lineData = token[line];
				String[] st = lineData.split(space);
				int offset = ("".equals(st[0])) ? 1 : 0;
				for (int position = 0; position < MAX_ORIENT; position++)
				{
					int num = Integer.parseInt(st[group * position + offset].trim());
					cubeLoc[face][position].face =
					startLoc[face][position].face = num;
					if (orientChoice) {
						num = Integer.parseInt(st[group * position + offset + 1].trim());
						cubeLoc[face][position].rotation =
						startLoc[face][position].rotation = num;
					}
				}
			}
			//blank line read
		} catch (NumberFormatException e) {
			//e.printStackTrace();
			if (incMoves)
				showMessage("Corrupt input in StartPosition");
			return -1;
		}
		return line;
	}

	boolean scanMoves(int location, int moves, boolean incMoves) {
		Graphics g = getGraphics();

		if (g == null)
			return false;
		try {
			int l;
			int line = location;
			String lineData;
			String symbol = SYMBOL;
			String space = "\\s+";

			++line; //read moves direction
			paint(g);
			for (l = 0; l < moves && ++line < token.length; l++) {
				lineData =  token[line];
				String[] st = lineData.substring(
					lineData.indexOf(symbol) + 2).split(space);
				currentDirection = Integer.parseInt(st[0].trim());
				currentStyle = Integer.parseInt(st[1].trim());
				currentControl = (Integer.parseInt(st[2].trim()) != 0);
				currentFace = Integer.parseInt(st[3].trim());
				currentPosition = Integer.parseInt(st[4].trim());
				{
					if (dim == 2 && !currentControl)
						dino2D.scanPaint = true;
					else if (dim == 3)
						dino3D.scanPaint = true;
					undo.setMove(currentDirection,
						currentStyle,
						(currentControl) ? 1 : 0,
						currentFace, currentPosition);
					redo.flushMoves();
					paint(g);
					if (incMoves && !currentControl)
						((DinoFrame) frame).callback(
						DinoInterface.ACTION_MOVED,
						dim);
				} /* else {
					currentDirection = -1;
					System.out.println(l +
						": move in direction " +
						currentDirection +
						", cannot be made.");
					return false;
				} */

			}
			if (l < moves) {
				if (incMoves)
					showMessage("Corrupt input in scanMoves, only " + l + " moves");
				return false;
			}
		} catch (NumberFormatException e) {
			//e.printStackTrace();
			if (incMoves)
				showMessage("Corrupt input in scanMoves");
			return false;
		} finally {
			g.dispose();
		}
		return true;
	}

	void printStartPosition(StringBuilder sb) {
		String symbol = SYMBOL;
		String space = " ";
		String newline = NL;

		sb.append(newline);
		sb.append("startingPosition").append(symbol).append(newline);
		for (int face = 0; face < MAX_FACES; face++) {
			for (int position = 0; position < MAX_ORIENT; position++)
			{
				int temp = startLoc[face][position].face;
				if (temp < 100)
					sb.append(space);
				if (temp < 10 && temp >= 0)
					sb.append(space);
				sb.append(space).append(temp);
				if (orientChoice) {
					temp = startLoc[face][position].rotation;
					if (temp < 10 && temp >= 0)
						sb.append(space);
					sb.append(space).append(temp);
				}
			}
			sb.append(newline);
		}
		sb.append(newline);
	}

	void printMoves(StringBuilder sb) {
		int direction, style, control;
		int face, position;
		int nMoves = undo.numMoves();
		String symbol = SYMBOL;
		String space = "\t";
		String newline = NL;

		sb.append("moves").append(space)
						.append("dir").append(space)
						.append("style").append(space)
						.append("control").append(space)
						.append("face").append(space)
						.append("position").append(newline);
		while (undo.madeMoves()) {
			undo.getMove();
			direction = undo.readMoveDirection();
			style = undo.readMoveStyle();
			control = undo.readMoveControl();
			face = undo.readMoveFace();
			position = undo.readMovePosition();
//System.out.println("direction " + direction + " style " + style + " control " +
//		control + "  " + " face " + face + " position " + position);
			redo.setMove(direction, style, control,
				face, position);
		}
		// apply all moves back to the stack since
		// we emptied it above!!
		for (int i = 0; i < nMoves; i++) {
			redo.getMove();
			direction = redo.readMoveDirection();
			style = redo.readMoveStyle();
			control = redo.readMoveControl();
			face = redo.readMoveFace();
			position = redo.readMovePosition();
			sb.append(i + 1).append(symbol).append(space)
								.append(direction).append(space)
								.append(style).append(space)
								.append(control).append(space)
								.append(face).append(space)
								.append(position).append(newline);
			undo.setMove(direction, style, control,
				face, position);
		}
	}

	@Override
	public void paint(Graphics g) {
		if (dim == 2)
			dino2D.paint(g);
		else if (dim == 3)
			dino3D.paint(g);
	}

	@Override
	public void update(Graphics g) {
		paint(g); /* no erase */
	}

	public boolean processKey(char key) {
		switch (key) {
		case '>':
		case '.':
			speedUpPuzzle();
			return true;
		case '<':
		case ',':
			slowDownPuzzle();
			return true;
		case '@':
			((DinoFrame) frame).toggleSound();
			return true;
		case '?':
			((DinoFrame) frame).descriptionHelp();
			return true;
		case '!':
			((DinoFrame) frame).featuresHelp();
			return true;
		case '^':
			((DinoFrame) frame).referencesHelp();
			return true;
		case 'a':
		case 'A':
			((DinoFrame) frame).aboutHelp();
			return true;
		case '*':
			((DinoFrame) frame).showBuffer();
			return true;
		default:
		}
		if (solve != null && solve.getFlag()) {
			solve.setFlag(false);
			return false;
		}
		switch (key) {
		case 'g':
			((DinoFrame) frame).getPuzzle();
			break;
		case 'w':
			writePuzzle();
			break;
		case 'G':
			((DinoFrame) frame).getPuzzleFile();
			break;
		case 'W':
			writePuzzleFile();
			break;
		case 'u':
		case 'U':
			undoPuzzle();
			break;
		case 'r':
		case 'R':
			redoPuzzle();
			break;
		case 'c':
		case 'C':
			clearPuzzle();
			break;
		case 'z':
		case 'Z':
			randomizePuzzle();
			break;
		case 'p':
		case 'P':
			practicePuzzle();
			break;
		case 's':
		case 'S':
			solvePuzzle();
			break;
		case 'o':
		case 'O':
			orientizePuzzle();
			break;
		case 'q':
		case 'Q':
		case 'x':
		case 'X':
		case '\03': /* Control-C */
			((DinoFrame) frame).quit();
			break;
		default:
			return false;
		}
		return true;
	}

	@Override
	public void mousePressed(MouseEvent event) {
		int modifiers = event.getModifiersEx();

		if (dino2D.scanPaint || dino3D.scanPaint ||
				dino2D.stackPaint || dino3D.stackPaint ||
				mouseDown)
			return;
		if (solve != null && solve.getFlag()) {
			solve.setFlag(false);
			return;
		}
		if (((modifiers & InputEvent.META_DOWN_MASK) == 0) &&
				((modifiers & InputEvent.ALT_DOWN_MASK) == 0)) {
			if (dino2D.randomPaints == 0 &&
					dino3D.randomPaints == 0) {
				selectPuzzle(((modifiers & InputEvent.CTRL_DOWN_MASK) != 0),
					event.getX(), event.getY());
				mouseDown = true;
			}
		} else {
			if ((event.getClickCount() & 1) == 1)
				randomizePuzzleWithQuery();
			else
				randomizePuzzle();
		}
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		int modifiers = event.getModifiersEx();

		if (mouseDown) { // Do not care which mouse button
			releasePuzzle(((modifiers & InputEvent.SHIFT_DOWN_MASK) != 0),
				((modifiers & InputEvent.CTRL_DOWN_MASK) != 0),
				((modifiers & InputEvent.ALT_DOWN_MASK) != 0),
				event.getX(), event.getY());
			mouseDown = false;
		}
	}

	@Override
	public void mouseEntered(MouseEvent event) {
		currentForeground = foreground;
		if (dim == 2) {
			/* dino2D.requestFocus(); */
			dino2D.framePaint = true;
		}
		else if (dim == 3) {
			/* dino3D.requestFocus(); */
			dino3D.framePaint = true;
		}
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseExited(MouseEvent event) {
		if (mouseDown && currentFace >= 0) {
			if (dim == 2)
				dino2D.releasePaint = true;
			else if (dim == 3)
				dino3D.releasePaint = true;
		}
		currentForeground = borderColor;
		if (dim == 2)
			dino2D.framePaint = true;
		else if (dim == 3)
			dino3D.framePaint = true;
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseDragged(MouseEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void mouseMoved(MouseEvent event) {
		lastX = event.getX();
		lastY = event.getY();
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent event) {
		int notches = event.getWheelRotation();
		int modifiers = event.getModifiersEx();

		if (notches < 0) {
			movePuzzleInput(lastX, lastY, TOP,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
		} else {
			movePuzzleInput(lastX, lastY, BOTTOM,
				(modifiers & InputEvent.SHIFT_DOWN_MASK) != 0,
				(modifiers & InputEvent.CTRL_DOWN_MASK) != 0,
				(modifiers & InputEvent.ALT_DOWN_MASK) != 0);
		}
	}

	@Override
	public void keyPressed(KeyEvent event) {
		if (event.getKeyCode() == KeyEvent.VK_ESCAPE)
			((DinoFrame) frame).shuffleDown();
		else if (!processKey(event.getKeyChar())) {
			if (dim == 2)
				dino2D.keyPressed(event);
			else if (dim == 3)
				dino3D.keyPressed(event);
		}
	}

	@Override
	public void keyReleased(KeyEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void keyTyped(KeyEvent event) {
		/* Method required but keyPressed does work */
	}
}

/**
 * The <code>DinoLoc</code> class maintains the locations of
 * the pieces.
 */

class DinoLoc {
	int	face;
	int	side;
	int	dir;

	public DinoLoc(int face, int side, int dir) {
		this.face = face;
		this.side = side;
		this.dir = dir;
	}
}

/**
 * The <code>DinoCornerLoc</code> class holds positional information
 * about the pieces.
 */

class DinoCornerLoc {
	int	face;
	int	rotation;

	public DinoCornerLoc(int face, int rotation) {
		this.face = face;
		this.rotation = rotation;
	}
}
