package com.sillycycle.bagleyd.dino;

/*
 * @(#)DinoSolve.java
 *
 * Copyright 2010 - 2023  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/* This puzzle is relatively easy as an edge cannot be flipped upsidedown. */
/* There are 2 different orientChoiceations possible when solved, red, yellow, */
/* blue in clockwise or counterclockwise order, in orientChoiceation mode, the */
/* order of the colors is clockwise. */

/* To swap from one orientChoiceation to other
1.  turn 4 non-overlapping corners clockwise (or counterclockwise),
2.  turn the other non-overlapping corners clockwise (or counterclockwise),
3.  repeat steps 1 and 2 now turning counterclockwise (clockwise) */

import java.util.Random;
import javax.swing.JFrame;

/**
 * The <code>DinoSolve</code> class is the code that does the work
 * of solving the puzzle.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/puzzles.html
 */

class DinoSolve {
	private static JFrame frame;
	private static Dino dino;
	private static final boolean DEBUG = false;
	private static Random generator;
	//private static int counter; // debugging

	DinoSolve(JFrame frame, Dino dino) {
		DinoSolve.frame = frame;
		DinoSolve.dino = dino;
		generator = new Random(System.nanoTime());
		//this.counter = 0;
	}

	boolean movePuzzleCorner(int face, int position, int direction,
			boolean control) {
		if (!getFlag()) {
			return false;
		}
		dino.movePuzzleDelay(face, position, direction,
			Dino.CORNER, control, Dino.DOUBLE);
		return true;
	}

	static final int MAX_CORNERS = 8;
	static final int MAX_EDGES = 12;

	static final int[][] dinoCorner /*[MAX_CORNERS=8][3]*/ =
	{
		{1, 4, 5},
		{1, 2, 4},
		{0, 1, 5},
		{0, 2, 1},
		{0, 5, 3},
		{3, 5, 4},
		{0, 3, 2},
		{2, 3, 4}
	};

	/*static final int[][] dinoCornerOrient =
	{
		{2, 2, 3},
		{1, 2, 3},
		{3, 3, 2},
		{2, 3, 0},
		{1, 0, 1},
		{1, 2, 0},
		{0, 1, 0},
		{1, 3, 0}
	};*/

	static final int[][] dinoEdge /*[MAX_EDGES][2]*/ =
	{
		{4, 5}, {2, 4}, {5, 0}, {0, 2},
		{1, 5}, {1, 2}, {3, 5}, {3, 2},
		{1, 4}, {1, 0}, {3, 4}, {3, 0}
	};

	static final int[][] dinoEdgeOrient /*[MAX_EDGES][2]*/ =
	{
		{2, 0}, {2, 0}, {2, 0}, {2, 0},
		{3, 3}, {1, 3}, {1, 1}, {3, 1},
		{2, 3}, {0, 3}, {2, 1}, {0, 1}
	};

	/* Corner layout, bits correspond to position
	   2 6
	 2 3 7 6
	 0 1 5 4
	   0 4
	   2 6
	  Edge layout
	     2
	   9  11
	     3
	4  5  10  6
	     1
	   8   7
	     0
	   4   6
	     2
	 */

	static int edgeColor(int edge, int n) {
		return dino.cubeLoc[dinoEdge[edge][n]][dinoEdgeOrient[edge][n]].face;
	}

	static int findCorner(boolean reverse, int colorFace1, int colorFace2) {
		int corner, face, i, j;

		if (reverse) {
			i = 2;
			j = 1;
		} else {
			i = 1;
			j = 2;
		}
		for (corner = 0; corner < MAX_CORNERS; corner++) {
			for (face = 0; face < 3; face++) {
				if (dinoCorner[corner][face] == colorFace1 &&
						dinoCorner[corner][(face + i) % 3] == colorFace2) {
					return dinoCorner[corner][(face + j) % 3];
				}
			}
		}
		return -1;
	}

	static int getEdgeIndex(int color0, int color1) {
		int i;

		for (i = 0; i < MAX_EDGES; i++) {
			if ((dinoEdge[i][0] == color0 && dinoEdge[i][1] == color1) ||
					(dinoEdge[i][1] == color0 && dinoEdge[i][0] == color1))
				return i;
		}
		return MAX_EDGES;
	}

	static int findEdge(int edge) {
		int i, j, t, count;
		int[] c = new int[2];

		c[0] = dinoEdge[edge][0];
		c[1] = dinoEdge[edge][1];
		for (i = 0; i < MAX_EDGES; i++) {
			count = 0;
			for (j = 0; j < 2; j++) {
				t = edgeColor(i, j);
				if (c[0] == t || c[1] == t)
					count++;
			}
			if (count == 2)
				return i;
		}
		return MAX_EDGES;
	}

	void setFirstEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(5, 0, Dino.BL, false);
			break;
		case 1:
			movePuzzleCorner(4, 0, Dino.BL, false);
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 3:
			movePuzzleCorner(2, 0, Dino.BL, false);
			movePuzzleCorner(1, 1, Dino.BL, false);
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 4:
			break;
		case 5:
			movePuzzleCorner(1, 1, Dino.BL, false);
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 6:
			movePuzzleCorner(5, 1, Dino.TL, false);
			movePuzzleCorner(5, 0, Dino.BL, false);
			break;
		case 7:
			movePuzzleCorner(2, 1, Dino.BL, false);
			movePuzzleCorner(4, 0, Dino.BL, false);
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 8:
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 9:
			movePuzzleCorner(1, 0, Dino.BR, false);
			movePuzzleCorner(1, 1, Dino.BL, false);
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 10:
			movePuzzleCorner(4, 1, Dino.BL, false);
			movePuzzleCorner(5, 0, Dino.BL, false);
			break;
		case 11:
			movePuzzleCorner(5, 2, Dino.TR, false);
			movePuzzleCorner(5, 2, Dino.TL, false);
			movePuzzleCorner(5, 1, Dino.BL, false);
			break;
		case 2:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setSecondEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(1, 0, Dino.BL, false);
			movePuzzleCorner(1, 3, Dino.BR, false);
			movePuzzleCorner(1, 3, Dino.TR, false);
			break;
		case 1:
			movePuzzleCorner(2, 2, Dino.TL, false);
			movePuzzleCorner(1, 1, Dino.TL, false);
			break;
		case 3:
			movePuzzleCorner(0, 2, Dino.TL, false);
			break;
		case 5:
			movePuzzleCorner(1, 1, Dino.TL, false);
			break;
		case 6:
			movePuzzleCorner(0, 3, Dino.TR, false);
			movePuzzleCorner(5, 1, Dino.BL, false);
			movePuzzleCorner(0, 0, Dino.BL, false);
			break;
		case 7:
			movePuzzleCorner(2, 1, Dino.TL, false);
			movePuzzleCorner(0, 2, Dino.TL, false);
			break;
		case 8:
			movePuzzleCorner(1, 2, Dino.TR, false);
			movePuzzleCorner(1, 1, Dino.TL, false);
			break;
		case 9:
			break;
		case 10:
			if (generator.nextInt(2) == 0) {
				movePuzzleCorner(4, 1, Dino.TL, false);
				movePuzzleCorner(2, 2, Dino.TL, false);
				movePuzzleCorner(1, 1, Dino.TL, false);
			} else {
				movePuzzleCorner(3, 2, Dino.TL, false);
				movePuzzleCorner(2, 1, Dino.TL, false);
				movePuzzleCorner(0, 2, Dino.TL, false);
			}
			break;
		case 11:
			movePuzzleCorner(0, 1, Dino.BL, false);
			movePuzzleCorner(0, 2, Dino.TL, false);
			break;
		case 2:
		case 4:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setThirdEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(5, 1, Dino.BL, false);
			movePuzzleCorner(5, 0, Dino.BR, false);
			movePuzzleCorner(5, 2, Dino.TR, false);
			break;
		case 1:
			movePuzzleCorner(2, 2, Dino.TR, false);
			movePuzzleCorner(3, 3, Dino.TR, false);
			break;
		case 3:
			movePuzzleCorner(0, 2, Dino.TR, false);
			break;
		case 5:
			movePuzzleCorner(2, 3, Dino.BR, false);
			movePuzzleCorner(2, 2, Dino.TR, false);
			movePuzzleCorner(3, 3, Dino.TR, false);
			break;
		case 6:
			movePuzzleCorner(3, 1, Dino.BL, false);
			movePuzzleCorner(3, 2, Dino.TL, false);
			movePuzzleCorner(3, 3, Dino.TR, false);
			break;
		case 7:
			movePuzzleCorner(3, 3, Dino.TR, false);
			break;
		case 8:
			movePuzzleCorner(4, 3, Dino.TR, false);
			movePuzzleCorner(2, 2, Dino.TR, false);
			movePuzzleCorner(3, 3, Dino.TR, false);
			break;
		case 10:
			movePuzzleCorner(3, 2, Dino.TL, false);
			movePuzzleCorner(3, 3, Dino.TR, false);
			break;
		case 11:
			break;
		case 2:
		case 4:
		case 9:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setForthEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(4, 2, Dino.TR, false);
			movePuzzleCorner(2, 0, Dino.BR, false);
			movePuzzleCorner(3, 2, Dino.TL, false);
			movePuzzleCorner(2, 1, Dino.TL, false);
			break;
		case 1:
			movePuzzleCorner(2, 0, Dino.BR, false);
			movePuzzleCorner(2, 2, Dino.TR, false);
			movePuzzleCorner(2, 1, Dino.TL, false);
			break;
		case 3:
			break;
		case 5:
			movePuzzleCorner(2, 3, Dino.BR, false);
			movePuzzleCorner(2, 0, Dino.BL, false);
			movePuzzleCorner(2, 2, Dino.TL, false);
			movePuzzleCorner(2, 3, Dino.TR, false);
			break;
		case 6:
			movePuzzleCorner(3, 1, Dino.BL, false);
			movePuzzleCorner(2, 0, Dino.BR, false);
			movePuzzleCorner(3, 2, Dino.TL, false);
			movePuzzleCorner(2, 1, Dino.TL, false);
			break;
		case 7:
			movePuzzleCorner(2, 1, Dino.BL, false);
			movePuzzleCorner(2, 0, Dino.BR, false);
			movePuzzleCorner(2, 2, Dino.TR, false);
			movePuzzleCorner(2, 1, Dino.TL, false);
			break;
		case 8:
			movePuzzleCorner(2, 0, Dino.BL, false);
			movePuzzleCorner(1, 2, Dino.TR, false);
			movePuzzleCorner(2, 3, Dino.TR, false);
			break;
		case 10:
			movePuzzleCorner(2, 0, Dino.BR, false);
			movePuzzleCorner(3, 2, Dino.TL, false);
			movePuzzleCorner(2, 1, Dino.TL, false);
			break;
		case 2:
		case 4:
		case 9:
		case 11:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setFifthEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(4, 2, Dino.TR, false);
			movePuzzleCorner(3, 2, Dino.TL, false);
			break;
		case 1:
			movePuzzleCorner(2, 2, Dino.TR, false);
			break;
		case 5:
			movePuzzleCorner(2, 3, Dino.BR, false);
			movePuzzleCorner(2, 2, Dino.TR, false);
			break;
		case 6:
			movePuzzleCorner(3, 1, Dino.BL, false);
			movePuzzleCorner(3, 2, Dino.TL, false);
			break;
		case 7:
			break;
		case 8:
			movePuzzleCorner(4, 3, Dino.TR, false);
			movePuzzleCorner(2, 2, Dino.TR, false);
			break;
		case 10:
			movePuzzleCorner(3, 2, Dino.TL, false);
			break;
		case 2:
		case 3:
		case 4:
		case 9:
		case 11:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setSixthEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(4, 2, Dino.TL, false);
			movePuzzleCorner(4, 0, Dino.BL, false);
			movePuzzleCorner(4, 3, Dino.BR, false);
			break;
		case 1:
			movePuzzleCorner(2, 2, Dino.TL, false);
			break;
		case 5:
			break;
		case 6:
			movePuzzleCorner(3, 1, Dino.BL, false);
			movePuzzleCorner(4, 1, Dino.TL, false);
			movePuzzleCorner(2, 2, Dino.TL, false);
			movePuzzleCorner(4, 0, Dino.BR, false);
			break;
		case 8:
			movePuzzleCorner(4, 0, Dino.BL, false);
			break;
		case 10:
			movePuzzleCorner(4, 1, Dino.TL, false);
			movePuzzleCorner(2, 2, Dino.TL, false);
			movePuzzleCorner(4, 0, Dino.BR, false);
			break;
		case 2:
		case 3:
		case 4:
		case 7:
		case 9:
		case 11:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setSeventhEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(4, 2, Dino.TR, false);
			movePuzzleCorner(4, 3, Dino.BR, false);
			movePuzzleCorner(4, 1, Dino.BL, false);
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 1:
			movePuzzleCorner(4, 1, Dino.TL, false);
			movePuzzleCorner(4, 3, Dino.TR, false);
			movePuzzleCorner(4, 0, Dino.BR, false);
			movePuzzleCorner(4, 0, Dino.BL, false);
			break;
		case 6:
			movePuzzleCorner(5, 0, Dino.BL, false);
			movePuzzleCorner(5, 1, Dino.TL, false);
			movePuzzleCorner(5, 3, Dino.TR, false);
			break;
		case 8:
			break;
		case 10:
			movePuzzleCorner(4, 3, Dino.BR, false);
			movePuzzleCorner(4, 1, Dino.BL, false);
			movePuzzleCorner(4, 2, Dino.TL, false);
			break;
		case 2:
		case 3:
		case 4:
		case 5:
		case 7:
		case 9:
		case 11:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setEighthEdge(int i) {
		int e = findEdge(i);

		if (DEBUG)
			System.out.println("nextEdge " + e + ", (" + i +  ")");
		switch (e) {
		case 0:
			movePuzzleCorner(4, 0, Dino.BR, false);
			movePuzzleCorner(4, 2, Dino.TR, false);
			movePuzzleCorner(4, 1, Dino.TL, false);
			break;
		case 1:
			break;
		case 6:
			movePuzzleCorner(4, 0, Dino.BR, false);
			movePuzzleCorner(4, 1, Dino.BL, false);
			movePuzzleCorner(4, 1, Dino.TL, false);
			break;
		case 10:
			movePuzzleCorner(4, 1, Dino.BL, false);
			movePuzzleCorner(4, 0, Dino.BR, false);
			movePuzzleCorner(4, 2, Dino.TR, false);
			movePuzzleCorner(4, 1, Dino.TL, false);
			break;
		case 2:
		case 3:
		case 4:
		case 5:
		case 7:
		case 8:
		case 9:
		case 11:
		default:
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong edge " + e);
		}
	}

	void setLastEdges() {
		int f;

		movePuzzleCorner(0, 1, Dino.TL, true);
		f = dino.cubeLoc[0][0].face;
		if (dino.cubeLoc[0][1].face == f) {
			return;
		}
		if (dino.cubeLoc[2][0].face == f) {
			movePuzzleCorner(2, 1, Dino.TL, false);
		} else {
			movePuzzleCorner(0, 1, Dino.BL, false);
		}
	}

	/* This red, yellow, blue in clockwise order unless reverse. */
	void solveEdges() {
		int[] faces = new int[6];
		boolean reverse = (dino.orientChoice) ? false :
			(generator.nextInt(2) == 0);

		faces[5] = dino.cubeLoc[5][2].face;
		faces[0] = dino.cubeLoc[0][0].face;
		faces[1] = findCorner(reverse, faces[5], faces[0]);
		if (DEBUG)
			System.out.println("zeroethEdge " + faces[5] +
				" " + faces[0] + " " + faces[1]);
		if (faces[1] < 0) {
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong face " + faces[1]);
			return;
		}
		setFirstEdge(getEdgeIndex(faces[1], faces[5]));
		setSecondEdge(getEdgeIndex(faces[0], faces[1]));
		faces[3] = findCorner(reverse, faces[0], faces[5]);
		if (faces[3] < 0) {
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong face " + faces[3]);
			return;
		}
		setThirdEdge(getEdgeIndex(faces[0], faces[3]));
		faces[2] = findCorner(reverse, faces[1], faces[0]);
		if (faces[2] < 0) {
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong face " + faces[2]);
			return;
		}
		setForthEdge(getEdgeIndex(faces[0], faces[2]));
		setFifthEdge(getEdgeIndex(faces[2], faces[3]));
		setSixthEdge(getEdgeIndex(faces[1], faces[2]));
		faces[4] = findCorner(reverse, faces[1], faces[2]);
		if (faces[4] < 0) {
			if (dino.period != DinoInterface.BOTH)
				System.out.println("Wrong face " + faces[4]);
			return;
		}
		setSeventhEdge(getEdgeIndex(faces[1], faces[4]));
		setEighthEdge(getEdgeIndex(faces[2], faces[4]));
		setLastEdges();
	}

	static final int[][] dinoFace /*[MAX_FACES=6][MAX_ORIENT=4]*/ =
	{
		{5, 3, 2, 1},
		{0, 2, 4, 5},
		{0, 3, 4, 1},
		{0, 5, 4, 2},
		{2, 3, 5, 1},
		{4, 3, 0, 1}
	};

	static final int[][] dinoFaceOrient /*[MAX_FACES=6][MAX_ORIENT=4]*/ =
	{
		{0, 1, 0, 1},
		{1, 0, 3, 2},
		{0, 0, 0, 0},
		{3, 2, 1, 0},
		{0, 3, 0, 3},
		{0, 2, 0, 2}
	};

	boolean movePuzzleEdge(int face, int position, int direction,
			boolean control) {
		if (!getFlag()) {
			return false;
		}
		dino.movePuzzleDelay(face, position, direction,
			Dino.EDGE, control, Dino.DOUBLE);
		return true;
	}

	boolean movePuzzleFace(int face, int position, int direction,
			boolean control) {
		if (!getFlag()) {
			return false;
		}
		dino.movePuzzleDelay(face, position, direction,
			Dino.FACE, control, Dino.DOUBLE);
		return true;
	}

	/*
	int position[Dino.MAX_ORIENT][2];

	void findFacets(int colorFace) {
		int face, edge, i = 0;

		for (edge = 0; edge < Dino.MAX_ORIENT; edge++) {
			position[edge][0] = -1;
			position[edge][1] = -1;
		}
		for (face = 0; face < Dino.MAX_FACES; face++) {
			for (edge = 0; edge < Dino.MAX_ORIENT; edge++) {
				if (colorFace == dino.cubeLoc[face][edge].face) {
					position[i][0] = face;
					position[i][1] = edge;
					i++;
				}
				if (i == Dino.MAX_ORIENT)
					return;
			}
		}
	}

	void printPosition(void) {
		int edge;
		for (edge = 0; edge < Dino.MAX_ORIENT; edge++) {
			System.out.println("face %d, position %d\n",
				position[edge][0], position[edge][1]);
		}
	}
	*/

	void flipFace(int face) {
		int i;
		for (i = 0; i < 14; i++) {
			movePuzzleEdge(dinoFace[face][0],
				(dinoFaceOrient[face][0] + 1) % 4,
				(dinoFaceOrient[face][0] + Dino.BL) % 4, false);
			movePuzzleEdge(face, 0, Dino.BR, false);
			movePuzzleEdge(dinoFace[face][1],
				(dinoFaceOrient[face][1] + 3) % 4,
				(dinoFaceOrient[face][1] + Dino.TR) % 4, false);
		}
	}

	/* equivalent to a movePuzzleCorner,
	   so BOTH mode does not make puzzle harder, 28 moves */
	void turnCorner(int face, int position, int direction) {
		int i;

		if (dino.period == DinoInterface.BOTH) {
			movePuzzleCorner(face, position, direction, false);
			return;
		}
		switch (direction) {
		case 0:
			switch (position) {
			case 0:
			case 3:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(face, 0, Dino.BL, false);
					movePuzzleEdge(dinoFace[face][0],
						dinoFaceOrient[face][0],
						(dinoFaceOrient[face][0] + Dino.BL) % 4, false);
					movePuzzleEdge(face, 3, Dino.BR, false);
					movePuzzleEdge(dinoFace[face][2],
						dinoFaceOrient[face][2],
						(dinoFaceOrient[face][2] + Dino.BL) % 4, false);
				}
				break;
			case 1:
			case 2:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(dinoFace[face][2],
						(dinoFaceOrient[face][2] + 1) % 4,
						(dinoFaceOrient[face][2] + Dino.TL) % 4, false);
					movePuzzleEdge(face, 1, Dino.TL, false);
					movePuzzleEdge(dinoFace[face][0],
						(dinoFaceOrient[face][0] + 1) % 4,
						(dinoFaceOrient[face][0] + Dino.TL) % 4, false);
					movePuzzleEdge(face, 2, Dino.TR, false);
				}
				break;
			}
			break;
		case 1:
			switch (position) {
			case 0:
			case 1:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(dinoFace[face][2],
						dinoFaceOrient[face][2],
						(dinoFaceOrient[face][2] + Dino.BR) % 4, false);
					movePuzzleEdge(face, 1, Dino.BL, false);
					movePuzzleEdge(dinoFace[face][0],
						dinoFaceOrient[face][0],
						(dinoFaceOrient[face][0] + Dino.BR) % 4, false);
					movePuzzleEdge(face, 0, Dino.BR, false);
				}
				break;
			case 2:
			case 3:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(face, 3, Dino.BR, false);
					movePuzzleEdge(dinoFace[face][2],
						(dinoFaceOrient[face][2] + 3) % 4,
						(dinoFaceOrient[face][2] + Dino.BR) % 4, false);
					movePuzzleEdge(face, 3, Dino.BL, false);
					movePuzzleEdge(dinoFace[face][2],
						(dinoFaceOrient[face][2] + 3) % 4,
						(dinoFaceOrient[face][2] + Dino.BL) % 4, false);
				}
				break;
			}
			break;
		case 2:
			switch (position) {
			case 0:
			case 3:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(dinoFace[face][2],
						dinoFaceOrient[face][2],
						(dinoFaceOrient[face][2] + Dino.BL) % 4, false);
					movePuzzleEdge(face, 3, Dino.BR, false);
					movePuzzleEdge(dinoFace[face][0],
						dinoFaceOrient[face][0],
						(dinoFaceOrient[face][0] + Dino.BL) % 4, false);
					movePuzzleEdge(face, 0, Dino.BL, false);
				}
				break;
			case 1:
			case 2:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(face, 2, Dino.TR, false);
					movePuzzleEdge(dinoFace[face][0],
						(dinoFaceOrient[face][0] + 1) % 4,
						(dinoFaceOrient[face][0] + Dino.TL) % 4, false);
					movePuzzleEdge(face, 1, Dino.TL, false);
					movePuzzleEdge(dinoFace[face][2],
						(dinoFaceOrient[face][2] + 1) % 4,
						(dinoFaceOrient[face][2] + Dino.TL) % 4, false);
				}
				break;
			}
			break;
		case 3:
			switch (position) {
			case 0:
			case 1:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(face, 0, Dino.BR, false);
					movePuzzleEdge(dinoFace[face][0],
						dinoFaceOrient[face][0],
						(dinoFaceOrient[face][0] + Dino.BR) % 4, false);
					movePuzzleEdge(face, 1, Dino.BL, false);
					movePuzzleEdge(dinoFace[face][2],
						dinoFaceOrient[face][2],
						(dinoFaceOrient[face][2] + Dino.BR) % 4, false);
				}
				break;
			case 2:
			case 3:
				for (i = 0; i < 7; i++) {
					movePuzzleEdge(dinoFace[face][2],
						(dinoFaceOrient[face][2] + 3) % 4,
						(dinoFaceOrient[face][2] + Dino.BL) % 4, false);
					movePuzzleEdge(face, 3, Dino.BL, false);
					movePuzzleEdge(dinoFace[face][2],
						(dinoFaceOrient[face][2] + 3) % 4,
						(dinoFaceOrient[face][2] + Dino.BR) % 4, false);
					movePuzzleEdge(face, 3, Dino.BR, false);
				}
				break;
			}
			break;
		}
	}

	/* face and position of lone changed piece, next piece CW or CCW to that,
	  repeat 4 times so no orientChoiceation change, 24 or 192 moves */
	void oppSwap(int face, int position, int direction,
			boolean repeat) {
		int i;
		switch (direction) {
		case Dino.CW:
			switch (face) {
			case 0:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);

						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);

						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);

						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);

						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);

						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);

						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);

						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);

						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);

						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);

						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);

						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);

						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
					}
					break;
				}
				break;
			case 1:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);

						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);

						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);

						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);

						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);

						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);

						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);

						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);

						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);

						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);

						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);

						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);

						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
					}
					break;
				}
				break;
			case 2:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);

						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);

						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);

						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);

						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);

						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);

						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);

						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);

						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);

						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);

						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);

						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);

						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
					}
					break;
				}
				break;
			case 3:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);

						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);

						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);

						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);

						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);

						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);

						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);

						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);

						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);

						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);

						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);

						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);

						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
					}
					break;
				}
				break;
			case 4:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);

						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);

						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);

						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);

						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);

						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);

						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);

						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);

						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);

						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);

						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);

						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);

						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
					}
					break;
				}
				break;
			case 5:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);

						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);

						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);

						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);

						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);

						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);

						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);

						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);

						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);

						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);

						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);

						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);

						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
					}
					break;
				}
				break;
			}
			break;
		case Dino.CCW:
			switch (face) {
			case 0:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);

						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);

						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);

						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);

						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);

						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);

						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);

						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);

						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);

						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);

						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);

						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);

						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
					}
					break;
				}
				break;
			case 1:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);

						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);

						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);

						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);

						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);

						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);

						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);

						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);

						movePuzzleEdge(1, 2, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);

						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);

						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);

						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);

						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
					}
					break;
				}
				break;
			case 2:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);

						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);

						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);

						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);

						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);

						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);

						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);

						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);

						movePuzzleEdge(2, 2, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);

						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);

						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);

						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);

						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
					}
					break;
				}
				break;
			case 3:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);

						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);

						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);

						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(2, 1, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);

						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);

						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);

						movePuzzleEdge(2, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(2, 3, Dino.BR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);

						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);

						movePuzzleEdge(3, 2, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);

						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);

						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);

						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.BR, false);
						movePuzzleEdge(0, 1, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);

						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.BR, false);
						movePuzzleEdge(0, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
					}
					break;
				}
				break;
			case 4:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);

						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);

						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);

						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);

						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);

						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);

						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(2, 2, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);

						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);

						movePuzzleEdge(4, 2, Dino.BR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(1, 2, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);

						movePuzzleEdge(2, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.TR, false);
						movePuzzleEdge(1, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(2, 0, Dino.TR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(4, 3, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(3, 2, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);

						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);

						movePuzzleEdge(4, 3, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(4, 1, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(4, 3, Dino.TR, false);

						movePuzzleEdge(3, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(3, 0, Dino.TR, false);
						movePuzzleEdge(4, 1, Dino.BR, false);
						movePuzzleEdge(3, 0, Dino.BR, false);
					}
					break;
				}
				break;
			case 5:
				switch (position) {
				case 0:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);

						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);

						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);

						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
					}
					break;
				case 1:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);

						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);

						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);

						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
					}
					break;
				case 2:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 2, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(4, 2, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);

						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);

						movePuzzleEdge(5, 2, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.TR, false);
						movePuzzleEdge(1, 3, Dino.TR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(5, 2, Dino.TR, false);

						movePuzzleEdge(4, 0, Dino.TR, false);
						movePuzzleEdge(1, 1, Dino.BR, false);
						movePuzzleEdge(1, 1, Dino.TR, false);
						movePuzzleEdge(4, 0, Dino.BR, false);
						movePuzzleEdge(5, 0, Dino.BR, false);
						movePuzzleEdge(4, 0, Dino.TR, false);
					}
					break;
				case 3:
					for (i = 0; i < ((repeat) ? 8 : 2); i++) {
						movePuzzleEdge(5, 3, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(3, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);

						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);

						movePuzzleEdge(5, 3, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.TR, false);
						movePuzzleEdge(0, 0, Dino.TR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(5, 3, Dino.TR, false);

						movePuzzleEdge(3, 3, Dino.TR, false);
						movePuzzleEdge(0, 2, Dino.BR, false);
						movePuzzleEdge(0, 2, Dino.TR, false);
						movePuzzleEdge(3, 3, Dino.BR, false);
						movePuzzleEdge(5, 1, Dino.BR, false);
						movePuzzleEdge(3, 3, Dino.TR, false);
					}
					break;
				}
				break;
			}
			break;
		}
	}

	/* 6 moves + 1 control move */
	void swapAdjFaces(int face0, int face1)
	{
		if ((face0 == 0 && face1 == 1) ||
				(face0 == 1 && face1 == 0)) {
			movePuzzleEdge(2, 0, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.BR, false);
			movePuzzleEdge(0, 0, Dino.BR, false);
			movePuzzleEdge(0, 2, Dino.TR, false);
			movePuzzleEdge(0, 2, Dino.BR, false);
			movePuzzleEdge(1, 0, Dino.BR, false);
			movePuzzleEdge(0, 0, Dino.BR, true);
		} else if ((face0 == 0 && face1 == 2) ||
				(face0 == 2 && face1 == 0)) {
			movePuzzleEdge(1, 0, Dino.BR, false);
			movePuzzleEdge(1, 0, Dino.TR, false);
			movePuzzleEdge(5, 2, Dino.BR, false);
			movePuzzleEdge(5, 2, Dino.TR, false);
			movePuzzleEdge(5, 0, Dino.BR, false);
			movePuzzleEdge(4, 0, Dino.TR, false);
			movePuzzleEdge(5, 2, Dino.BR, true);
		} else if ((face0 == 0 && face1 == 3) ||
				(face0 == 3 && face1 == 0)) {
			movePuzzleEdge(2, 0, Dino.BR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(0, 0, Dino.TR, false);
			movePuzzleEdge(0, 2, Dino.BR, false);
			movePuzzleEdge(0, 2, Dino.TR, false);
			movePuzzleEdge(1, 0, Dino.BR, false);
			movePuzzleEdge(0, 0, Dino.TR, true);
		} else if ((face0 == 0 && face1 == 5) ||
				(face0 == 5 && face1 == 0)) {
			movePuzzleEdge(1, 3, Dino.TR, false);
			movePuzzleEdge(1, 3, Dino.BR, false);
			movePuzzleEdge(4, 2, Dino.BR, false);
			movePuzzleEdge(4, 2, Dino.TR, false);
			movePuzzleEdge(4, 0, Dino.BR, false);
			movePuzzleEdge(2, 0, Dino.TR, false);
			movePuzzleEdge(4, 2, Dino.BR, true);
		} else if ((face0 == 1 && face1 == 2) ||
				(face0 == 2 && face1 == 1)) {
			movePuzzleEdge(0, 2, Dino.BR, false);
			movePuzzleEdge(0, 0, Dino.TR, false);
			movePuzzleEdge(2, 0, Dino.BR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.BR, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 0, Dino.BR, true);
		} else if ((face0 == 1 && face1 == 4) ||
				(face0 == 4 && face1 == 1)) {
			movePuzzleEdge(5, 0, Dino.TR, false);
			movePuzzleEdge(5, 2, Dino.BR, false);
			movePuzzleEdge(0, 2, Dino.TR, false);
			movePuzzleEdge(0, 0, Dino.BR, false);
			movePuzzleEdge(0, 0, Dino.TR, false);
			movePuzzleEdge(1, 3, Dino.BR, false);
			movePuzzleEdge(0, 2, Dino.TR, true);
		} else if ((face0 == 1 && face1 == 5) ||
				(face0 == 5 && face1 == 1)) {
			movePuzzleEdge(0, 0, Dino.TR, false);
			movePuzzleEdge(0, 2, Dino.BR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(2, 0, Dino.BR, false);
			movePuzzleEdge(2, 0, Dino.TR, false);
			movePuzzleEdge(1, 0, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, true);
		} else if ((face0 == 2 && face1 == 3) ||
				(face0 == 3 && face1 == 2)) {
			movePuzzleEdge(0, 2, Dino.TR, false);
			movePuzzleEdge(0, 0, Dino.BR, false);
			movePuzzleEdge(2, 0, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.BR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 0, Dino.TR, true);
		} else if ((face0 == 2 && face1 == 4) ||
				(face0 == 4 && face1 == 2)) {
			movePuzzleEdge(1, 1, Dino.TR, false);
			movePuzzleEdge(1, 1, Dino.BR, false);
			movePuzzleEdge(0, 2, Dino.BR, false);
			movePuzzleEdge(0, 2, Dino.TR, false);
			movePuzzleEdge(0, 0, Dino.BR, false);
			movePuzzleEdge(5, 0, Dino.TR, false);
			movePuzzleEdge(0, 2, Dino.BR, true);
		} else if ((face0 == 3 && face1 == 4) ||
				(face0 == 4 && face1 == 3)) {
			movePuzzleEdge(5, 0, Dino.BR, false);
			movePuzzleEdge(5, 2, Dino.TR, false);
			movePuzzleEdge(0, 2, Dino.BR, false);
			movePuzzleEdge(0, 0, Dino.TR, false);
			movePuzzleEdge(0, 0, Dino.BR, false);
			movePuzzleEdge(3, 1, Dino.TR, false);
			movePuzzleEdge(0, 2, Dino.BR, true);
		} else if ((face0 == 3 && face1 == 5) ||
				(face0 == 5 && face1 == 3)) {
			movePuzzleEdge(0, 0, Dino.BR, false);
			movePuzzleEdge(0, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.BR, false);
			movePuzzleEdge(2, 0, Dino.TR, false);
			movePuzzleEdge(2, 0, Dino.BR, false);
			movePuzzleEdge(1, 0, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.BR, true);
		} else if ((face0 == 4 && face1 == 5) ||
				(face0 == 5 && face1 == 4)) {
			movePuzzleEdge(1, 2, Dino.BR, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.BR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(2, 0, Dino.BR, false);
			movePuzzleEdge(0, 0, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.BR, true);
		}
	}

	void rotatePairs(int face0, int face1)
	{
		switch (face0) {
		case 0:
			switch(face1) {
			case 1:
				/* Dino.CCW */
				swapAdjFaces(2, 3);
				swapAdjFaces(3, 4);
				swapAdjFaces(2, 4);
				swapAdjFaces(3, 4);
				break;
			case 3:
				/* Dino.CW */
				swapAdjFaces(1, 2);
				swapAdjFaces(1, 4);
				swapAdjFaces(2, 4);
				swapAdjFaces(1, 4);
				break;
			}
			break;
		case 1:
			switch(face1) {
			case 4:
				/* Dino.CW */
				swapAdjFaces(2, 3);
				swapAdjFaces(0, 3);
				swapAdjFaces(0, 2);
				swapAdjFaces(0, 3);
				break;
			}
			break;
		case 3:
			switch(face1) {
			case 4:
				/* Dino.CCW */
				swapAdjFaces(1, 2);
				swapAdjFaces(0, 1);
				swapAdjFaces(0, 2);
				swapAdjFaces(0, 1);
				break;
			}
			break;
		}
	}

	void balancedRotate() {
		int i;
		for (i = 0; i< 5; i++) {
			movePuzzleEdge(2, 0, Dino.TL, false);
			movePuzzleEdge(0, 1, Dino.TR, false);
			movePuzzleEdge(3, 3, Dino.BL, false);
			movePuzzleEdge(2, 3, Dino.TR, false);
			movePuzzleEdge(0, 2, Dino.TL, false);
			movePuzzleEdge(1, 0, Dino.BR, false);
		}
	}

	void flipLR(int face) {
		movePuzzleEdge(face, 0, Dino.TR, false);
		movePuzzleEdge(face, 0, Dino.TL, false);
		movePuzzleEdge(face, 0, Dino.TR, false);
	}

	/*
	void flipTB(int face) {
		movePuzzleEdge(face, 1, Dino.TR, false);
		movePuzzleEdge(face, 1, Dino.BR, false);
		movePuzzleEdge(face, 1, Dino.TR, false);
	}

	void adjacentSplit40(int face) {
		int i;
		for (i = 0; i < 40; i++) {
			movePuzzleEdge(dinoFace[face][0],
				dinoFaceOrient[face][0],
				(dinoFaceOrient[face][0] + Dino.TR) % 4, false);
			movePuzzleEdge(5, 0, Dino.TR, false);
			movePuzzleEdge(dinoFace[face][0],
				(dinoFaceOrient[face][0] + 3) % 4,
				(dinoFaceOrient[face][0] + Dino.TL) % 4, false);
			movePuzzleEdge(dinoFace[face][0],
				(dinoFaceOrient[face][0] + 1) % 4,
				(dinoFaceOrient[face][0] + Dino.BL) % 4, false);
			movePuzzleEdge(5, 0, Dino.BR, false);
			movePuzzleEdge(dinoFace[face][0],
				dinoFaceOrient[face][0],
				(dinoFaceOrient[face][0] + Dino.BR) % 4, false);
		}
	}

	void adjacentSplit(int face) {
		int i;
		for (i = 0; i < 3; i++) {
			movePuzzleEdge(face, 1, Dino.BL, false);
			movePuzzleEdge(face, 0, Dino.BR, false);
			movePuzzleEdge(dinoFace[face][0],
				dinoFaceOrient[face][0],
				(dinoFaceOrient[face][0] + Dino.BR) % 4, false);
			if (i == 2)
				break;
			movePuzzleEdge(face, 0, Dino.BR, false);
		}
	}

	void spinOut(int face) {
		int i, j;
		for (i = 0; i < 2; i++) {
			for (j = 0; j < 2; j++) {
				movePuzzleEdge(dinoFace[face][0],
					(dinoFaceOrient[face][0] + 3) % 4,
					(dinoFaceOrient[face][0] + Dino.BR) % 4, false);
				movePuzzleEdge(dinoFace[face][3],
					dinoFaceOrient[face][3],
					(dinoFaceOrient[face][3] + Dino.BR) % 4, false);
			}
			for (j = 0; j < 2; j++) {
				movePuzzleEdge(dinoFace[face][1],
					(dinoFaceOrient[face][1] + 2) % 4,
					(dinoFaceOrient[face][1] + Dino.TL) % 4, false);
				movePuzzleEdge(dinoFace[face][2],
					(dinoFaceOrient[face][2] + 1) % 4,
					(dinoFaceOrient[face][2] + Dino.TL) % 4, false);
			}
		}
	}

	void fiveOut(int face) {
		int i;
		for (i = 0; i < 10; i++) {
			movePuzzleEdge(dinoFace[face][0],
				(dinoFaceOrient[face][0] + 3) % 4,
				(dinoFaceOrient[face][0] + Dino.BR) % 4, false);
			movePuzzleEdge(face, 0, Dino.BR, false);
			movePuzzleEdge(dinoFace[face][2],
				dinoFaceOrient[face][2],
				(dinoFaceOrient[face][2] + Dino.BR) % 4, false);
		}
	}

	void turnCube(int face) {
		movePuzzleEdge(face, 2, Dino.TL, false);
		movePuzzleEdge(dinoFace[face][2],
			dinoFaceOrient[face][2],
			(dinoFaceOrient[face][2] + Dino.BL) % 4, false);
		movePuzzleEdge(dinoFace[face][0],
			(dinoFaceOrient[face][0] + 1) % 4,
			(dinoFaceOrient[face][0] + Dino.TL) % 4, false);
		movePuzzleEdge(dinoFace[face][3],
			(dinoFaceOrient[face][3] + 2) % 4,
			(dinoFaceOrient[face][3] + Dino.TL) % 4, false);
	}
	*/

	/* first part of oppSwap(0, 0, CW), result similar to oppSwap(4, 2, CW) */
	void topFaceFromRightSide() {
		movePuzzleEdge(0, 0, Dino.BR, false);
		movePuzzleEdge(2, 2, Dino.TR, false);
		movePuzzleEdge(0, 0, Dino.TR, false);
		movePuzzleEdge(2, 0, Dino.BR, false);
		movePuzzleEdge(0, 2, Dino.BR, false);
		movePuzzleEdge(0, 2, Dino.TR, false);

		movePuzzleEdge(1, 2, Dino.BR, false);
		movePuzzleEdge(2, 2, Dino.TR, false);
		movePuzzleEdge(2, 2, Dino.BR, false);
		movePuzzleEdge(1, 2, Dino.TR, false);
		movePuzzleEdge(0, 2, Dino.BR, false);
		movePuzzleEdge(1, 2, Dino.BR, false);
	}

	/* first part of oppSwap(0, 0, CCW), result similar to oppSwap(4, 2, CCW) */
	void topFaceFromLeftSide() {
		movePuzzleEdge(0, 0, Dino.TR, false);
		movePuzzleEdge(2, 2, Dino.BR, false);
		movePuzzleEdge(0, 0, Dino.BR, false);
		movePuzzleEdge(2, 0, Dino.TR, false);
		movePuzzleEdge(0, 2, Dino.TR, false);
		movePuzzleEdge(0, 2, Dino.BR, false);

		movePuzzleEdge(3, 2, Dino.TR, false);
		movePuzzleEdge(2, 2, Dino.BR, false);
		movePuzzleEdge(2, 2, Dino.TR, false);
		movePuzzleEdge(3, 2, Dino.BR, false);
		movePuzzleEdge(0, 2, Dino.TR, false);
		movePuzzleEdge(3, 2, Dino.TR, false);
	}

	void setFace0Pos1(int color) {
		int face = 0;
		int pos = 1;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[4][0].face == color) {
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[0][3].face == color) {
			movePuzzleEdge(0, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[1][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[3][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[5][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[1][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][1].face == color) {
			movePuzzleEdge(0, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][3].face == color) {
			movePuzzleEdge(0, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[3][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[5][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[5][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[1][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][0].face == color) {
			movePuzzleEdge(0, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[3][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[4][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[4][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[0][2].face == color) {
			movePuzzleEdge(0, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[1][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[3][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[4][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[5][0].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace0Pos3(int color) {
		int face = 0;
		int pos = 3;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[4][0].face == color) {
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[1][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[2][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[3][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[5][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[1][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[2][1].face == color) {
			movePuzzleEdge(0, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[3][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[5][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[5][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[1][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[3][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[4][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[4][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[0][2].face == color) {
			movePuzzleEdge(0, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[1][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[2][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[3][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[4][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[2][0].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[5][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace0Pos2(int color) {
		int face = 0;
		int pos = 2;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[4][1].face == color) {
			topFaceFromRightSide();
		} else if (dino.cubeLoc[4][3].face == color) {
			topFaceFromLeftSide();
		} else if (dino.cubeLoc[1][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[1][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			topFaceFromLeftSide();
		} else if (dino.cubeLoc[3][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromLeftSide();
		} else if (dino.cubeLoc[2][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			topFaceFromLeftSide();
		} else if (dino.cubeLoc[2][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[3][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[5][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			topFaceFromLeftSide();
		} else if (dino.cubeLoc[5][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[1][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[1][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[3][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			topFaceFromLeftSide();
		} else if (dino.cubeLoc[3][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			topFaceFromLeftSide();
		} else if (dino.cubeLoc[4][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[4][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[2][0].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[2][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[5][0].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else if (dino.cubeLoc[5][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TL, false);
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			topFaceFromRightSide();
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	} 

	void setFace1Pos3(int color) {
		int face = 1;
		int pos = 3;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[3][2].face == color) {
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[1][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[4][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[2][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[5][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[4][3].face == color) {
			turnCorner(4, 3, Dino.TL);
		} else if (dino.cubeLoc[5][0].face == color) {
			turnCorner(5, 0, Dino.BL);
		} else if (dino.cubeLoc[2][1].face == color) {
			turnCorner(2, 1, Dino.TR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[3][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 3, Dino.TL);
		} else if (dino.cubeLoc[4][0].face == color) {
			turnCorner(4, 0, Dino.BR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[4][2].face == color) {
			turnCorner(4, 2, Dino.TR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[5][1].face == color) {
			turnCorner(5, 1, Dino.BR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[2][0].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			turnCorner(4, 2, Dino.TR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[2][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.BR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[3][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.BR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[3][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 2, Dino.TR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[5][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TL, false);
			turnCorner(4, 0, Dino.BR);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else if (dino.cubeLoc[1][2].face == color) {
			turnCorner(2, 3, Dino.BR);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TL, false);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace1Pos2(int color) {
		int face = 1;
		int pos = 2;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[3][3].face == color) {
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[4][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][3].face == color) {
			turnCorner(2, 2, Dino.TL);
		} else if (dino.cubeLoc[4][0].face == color) {
			turnCorner(4, 3, Dino.BL);
		} else if (dino.cubeLoc[2][0].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][2].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			turnCorner(4, 3, Dino.BL);
		} else if (dino.cubeLoc[3][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.BL);
		} else if (dino.cubeLoc[4][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			turnCorner(2, 2, Dino.TL);
		} else if (dino.cubeLoc[3][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.BR);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[5][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(2, 2, Dino.TR);
			movePuzzleEdge(1, 2, Dino.TR, false);
		} else if (dino.cubeLoc[1][1].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.BR);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][1].face == color) {
			turnCorner(2, 1, Dino.BL);
			turnCorner(4, 3, Dino.BL);
		} else if (dino.cubeLoc[4][3].face == color) {
			movePuzzleEdge(1, 2, Dino.TR, false);
			turnCorner(2, 1, Dino.BL);
			turnCorner(4, 3, Dino.BL);
		} else if (dino.cubeLoc[5][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(2, 1, Dino.BL);
			turnCorner(4, 3, Dino.BL);
		} else if (dino.cubeLoc[3][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(1, 2, Dino.TR, false);
			turnCorner(2, 1, Dino.BL);
			turnCorner(4, 3, Dino.BL);
		} else if (dino.cubeLoc[5][3].face == color) {
		 	oppSwap(2, 0, Dino.CW, true);
			movePuzzleEdge(1, 2, Dino.TR, false);
			movePuzzleEdge(2, 2, Dino.TR, false);
			movePuzzleEdge(face, 2, Dino.TR, false);
		} else if (dino.cubeLoc[5][2].face == color) {
		 	oppSwap(2, 3, Dino.CW, true);
			turnCorner(2, 2, Dino.TL);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace1Pos1(int color) {
		int face = 1;
		int pos = 1;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[3][0].face == color) {
		 	oppSwap(1, 1, Dino.CW, false);
		} else if (dino.cubeLoc[3][2].face == color) {
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[4][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[4][3].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
		 	oppSwap(1, 1, Dino.CW, false);
		} else if (dino.cubeLoc[2][1].face == color) {
			turnCorner(2, 1, Dino.TR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[4][0].face == color) {
			turnCorner(4, 0, Dino.BR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[4][2].face == color) {
			turnCorner(4, 1, Dino.TR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[5][1].face == color) {
			turnCorner(5, 0, Dino.BR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[3][1].face == color) {
			turnCorner(4, 1, Dino.BL);
			movePuzzleEdge(2, 2, Dino.TR, false);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[3][3].face == color) {
			turnCorner(4, 1, Dino.TL);
			movePuzzleEdge(2, 2, Dino.TR, false);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[5][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(2, 1, Dino.TR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[5][2].face == color) {
		 	oppSwap(2, 1, Dino.CCW, true);
			turnCorner(2, 1, Dino.TR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[2][0].face == color) {
		 	oppSwap(5, 1, Dino.CW, true);
			turnCorner(5, 0, Dino.BR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[2][2].face == color) {
		 	oppSwap(5, 1, Dino.CCW, true);
			turnCorner(5, 0, Dino.BR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[2][3].face == color) {
		 	oppSwap(5, 0, Dino.CCW, true); /* across face */
			turnCorner(2, 1, Dino.TR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[5][3].face == color) {
		 	oppSwap(2, 2, Dino.CW, true); /* across face */
			turnCorner(5, 0, Dino.BR);
		 	oppSwap(1, 1, Dino.CCW, false);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace2Pos3(int color) {
		int face = 2;
		int pos = 3;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[2][1].face == color) {
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[5][2].face == color) {
		 	oppSwap(5, 2, Dino.CW, true);
		} else if (dino.cubeLoc[5][0].face == color) {
		 	oppSwap(5, 0, Dino.CCW, true);
		} else if (dino.cubeLoc[4][0].face == color) {
			turnCorner(4, 0, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[3][1].face == color) {
			turnCorner(3, 1, Dino.TR);
		 	oppSwap(5, 0, Dino.CCW, true);
		} else if (dino.cubeLoc[3][2].face == color) {
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[4][2].face == color) {
			turnCorner(4, 1, Dino.TR);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[5][1].face == color) {
			turnCorner(5, 0, Dino.BR);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[4][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[2][2].face == color) {
			turnCorner(2, 1, Dino.BL);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[3][3].face == color) {
			turnCorner(3, 2, Dino.BR);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[3][0].face == color) {
			flipFace(3);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[4][3].face == color) {
			flipFace(4);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else if (dino.cubeLoc[5][3].face == color) {
			flipFace(5);
			turnCorner(5, 0, Dino.BR);
			turnCorner(3, 2, Dino.TL);
		 	oppSwap(5, 0, Dino.CW, true);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace2Pos2(int color) {
		int face = 2;
		int pos = 2;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[5][1].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
		} else if (dino.cubeLoc[3][3].face == color) {
			turnCorner(3, 2, Dino.TL);
		} else if (dino.cubeLoc[4][1].face == color) {
			turnCorner(4, 0, Dino.TL);
		} else if (dino.cubeLoc[4][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(3, 2, Dino.TL);
		} else if (dino.cubeLoc[3][2].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.TL);
		} else if (dino.cubeLoc[3][1].face == color) {
			flipFace(3);
			turnCorner(3, 2, Dino.TL);
		} else if (dino.cubeLoc[4][3].face == color) {
			flipFace(4);
			turnCorner(4, 0, Dino.TL);
		} else if (dino.cubeLoc[5][3].face == color) {
			flipFace(5);
			movePuzzleEdge(2, 2, Dino.TR, false);
		} else if (dino.cubeLoc[2][1].face == color) {
			turnCorner(2, 1, Dino.TR);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.TL);
		} else if (dino.cubeLoc[3][0].face == color) {
			flipFace(3);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.TL);
		} else if (dino.cubeLoc[4][0].face == color) {
			flipFace(4);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(3, 2, Dino.TL);
		} else if (dino.cubeLoc[5][0].face == color) {
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(2, 1, Dino.TR);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.TL);
		} else if (dino.cubeLoc[5][2].face == color) {
			flipFace(5);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(2, 1, Dino.TR);
			movePuzzleEdge(2, 2, Dino.TR, false);
			turnCorner(4, 0, Dino.TL);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace2Pos1(int color) {
		int face = 2;
		int pos = 1;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[3][0].face == color) {
			balancedRotate();
		} else if (dino.cubeLoc[3][2].face == color) {
			flipFace(3);
			balancedRotate();
		} else if (dino.cubeLoc[4][2].face == color) {
			turnCorner(4, 1, Dino.TR);
			flipFace(3);
			balancedRotate();
		} else if (dino.cubeLoc[4][0].face == color) {
			flipFace(4);
			turnCorner(4, 1, Dino.TR);
			flipFace(3);
			balancedRotate();
		} else if (dino.cubeLoc[5][1].face == color) {
			turnCorner(5, 0, Dino.BR);
			flipFace(3);
			balancedRotate();
		} else if (dino.cubeLoc[5][3].face == color) {
			flipFace(5);
			turnCorner(5, 0, Dino.BR);
			flipFace(3);
			balancedRotate();
		} else if (dino.cubeLoc[5][0].face == color) {
		 	oppSwap(2, 1, Dino.CW, false);
		} else if (dino.cubeLoc[5][2].face == color) {
		 	oppSwap(2, 1, Dino.CCW, false);
		} else if (dino.cubeLoc[3][1].face == color) {
			turnCorner(3, 1, Dino.TR);
		 	oppSwap(2, 1, Dino.CW, false);
		} else if (dino.cubeLoc[4][1].face == color) {
			turnCorner(4, 1, Dino.BL);
		 	oppSwap(2, 1, Dino.CW, false);
		} else if (dino.cubeLoc[3][3].face == color) {
			flipFace(3);
			turnCorner(3, 1, Dino.TR);
		 	oppSwap(2, 1, Dino.CW, false);
		} else if (dino.cubeLoc[4][3].face == color) {
			flipFace(4);
			turnCorner(4, 1, Dino.BL);
		 	oppSwap(2, 1, Dino.CW, false);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace3Pos1(int color) {
		int face = 3;
		int pos = 1;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[3][2].face == color) {
			turnCorner(3, 1, Dino.BL);
			swapAdjFaces(1, 2);
			turnCorner(4, 1, Dino.TR);
		} else if (dino.cubeLoc[4][0].face == color) {
			swapAdjFaces(1, 2);
			flipFace(4);
			turnCorner(4, 1, Dino.TR);
		} else if (dino.cubeLoc[4][2].face == color) {
			swapAdjFaces(1, 2);
			turnCorner(4, 1, Dino.TR);
		} else if (dino.cubeLoc[3][3].face == color) {
			swapAdjFaces(2, 3);
		 	oppSwap(5, 0, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][1].face == color) {
			turnCorner(4, 1, Dino.TR);
		} else if (dino.cubeLoc[4][3].face == color) {
			flipFace(4);
			turnCorner(4, 1, Dino.TR);
		} else if (dino.cubeLoc[5][0].face == color) {
			turnCorner(5, 0, Dino.BR);
		} else if (dino.cubeLoc[5][1].face == color) {
			swapAdjFaces(0, 1);
			turnCorner(5, 0, Dino.BR);
		} else if (dino.cubeLoc[5][2].face == color) {
			swapAdjFaces(2, 3);
		 	oppSwap(2, 3, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[5][3].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 3, Dino.CW, false);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace3Pos2(int color) {
		int face = 3;
		int pos = 2;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[5][0].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 2, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[5][2].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 2, Dino.CCW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][0].face == color) {
			swapAdjFaces(0, 2);
			swapAdjFaces(2, 4);
			swapAdjFaces(4, 5);
			turnCorner(2, 1, Dino.TR);
			swapAdjFaces(4, 5);
			swapAdjFaces(2, 4);
			swapAdjFaces(0, 2);
		} else if (dino.cubeLoc[5][1].face == color) {
			swapAdjFaces(0, 2);
			swapAdjFaces(2, 4);
			swapAdjFaces(4, 5);
			turnCorner(4, 0, Dino.BR);
			swapAdjFaces(4, 5);
			swapAdjFaces(2, 4);
			swapAdjFaces(0, 2);
		} else if (dino.cubeLoc[4][2].face == color) {
			swapAdjFaces(0, 2);
			swapAdjFaces(2, 4);
			swapAdjFaces(4, 5);
			flipFace(2);
			turnCorner(2, 1, Dino.TR);
			swapAdjFaces(4, 5);
			swapAdjFaces(2, 4);
			swapAdjFaces(0, 2);
		} else if (dino.cubeLoc[5][3].face == color) {
			swapAdjFaces(0, 2);
			swapAdjFaces(2, 4);
			swapAdjFaces(4, 5);
			flipFace(4);
			turnCorner(4, 0, Dino.BR);
			swapAdjFaces(4, 5);
			swapAdjFaces(2, 4);
			swapAdjFaces(0, 2);
		} else if (dino.cubeLoc[3][3].face == color) {
			swapAdjFaces(2, 3);
		 	oppSwap(2, 1, Dino.CCW, true);
			swapAdjFaces(2, 3);
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 2, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][1].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(5, 0, Dino.CCW, true);
			swapAdjFaces(2, 4);
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 2, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][3].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(5, 0, Dino.CW, true);
			swapAdjFaces(2, 4);
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 2, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace3Pos3(int color) {
		int face = 3;
		int pos = 3;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[5][0].face == color) {
			swapAdjFaces(2, 3);
		 	oppSwap(2, 1, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[5][2].face == color) {
			swapAdjFaces(2, 3);
		 	oppSwap(2, 1, Dino.CCW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[5][1].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 1, Dino.CW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[5][3].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		 	oppSwap(2, 1, Dino.CCW, false);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][1].face == color) {
			swapAdjFaces(2, 3);
			swapAdjFaces(4, 5);
		 	oppSwap(2, 1, Dino.CCW, false);
			swapAdjFaces(4, 5);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][3].face == color) {
			swapAdjFaces(2, 3);
			swapAdjFaces(4, 5);
		 	oppSwap(2, 1, Dino.CW, false);
			swapAdjFaces(4, 5);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][0].face == color) {
			swapAdjFaces(2, 3);
			swapAdjFaces(0, 1);
			swapAdjFaces(4, 5);
		 	oppSwap(2, 2, Dino.CW, false);
			swapAdjFaces(4, 5);
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		} else if (dino.cubeLoc[4][2].face == color) {
			swapAdjFaces(2, 3);
			swapAdjFaces(0, 1);
			swapAdjFaces(4, 5);
		 	oppSwap(2, 2, Dino.CCW, false);
			swapAdjFaces(4, 5);
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 3);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace4Pos1(int color) {
		int face = 4;
		int pos = 1;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[5][0].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(2, 1, Dino.CW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][2].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(2, 1, Dino.CCW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][1].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 4);
		 	oppSwap(2, 1, Dino.CW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][3].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 4);
		 	oppSwap(2, 1, Dino.CCW, false);
			swapAdjFaces(2, 4);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace4Pos2(int color) {
		int face = 4;
		int pos = 2;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[5][1].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(2, 0, Dino.CCW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][3].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(2, 0, Dino.CW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][0].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 4);
		 	oppSwap(2, 0, Dino.CCW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][2].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 4);
		 	oppSwap(2, 0, Dino.CW, false);
			swapAdjFaces(2, 4);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace4Pos3(int color) {
		int face = 4;
		int pos = 3;

		if (dino.cubeLoc[face][pos].face == color) {
			return;
		} else if (dino.cubeLoc[5][0].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(2, 3, Dino.CCW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][2].face == color) {
			swapAdjFaces(2, 4);
		 	oppSwap(2, 3, Dino.CW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][1].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 4);
		 	oppSwap(2, 3, Dino.CCW, false);
			swapAdjFaces(2, 4);
		} else if (dino.cubeLoc[5][3].face == color) {
			swapAdjFaces(0, 1);
			swapAdjFaces(2, 4);
		 	oppSwap(2, 3, Dino.CW, false);
			swapAdjFaces(2, 4);
		} else {
			System.out.println("Fix face " + face + " pos " + pos);
		}
	}

	void setFace0() {
		int face = 0;
		int pos = 0;
		int color = dino.cubeLoc[face][pos].face;
		if (DEBUG)
			System.out.println("face " + face + ": zeroethEdge " + color);
		setFace0Pos1(color);
		setFace0Pos3(color);
		setFace0Pos2(color);
	}

	void setFace1() {
		int face = 1;
		int pos = 0;
		int color = dino.cubeLoc[face][pos].face;
		if (DEBUG)
			System.out.println("face " + face + ": zeroethEdge " + color);
		setFace1Pos3(color);
		setFace1Pos2(color);
		setFace1Pos1(color);
	}

	void setFace2() {
		int face = 2;
		int pos = 0;
		int color = dino.cubeLoc[face][pos].face;
		if (DEBUG)
			System.out.println("face " + face + ": zeroethEdge " + color);
		setFace2Pos3(color);
		setFace2Pos2(color);
		setFace2Pos1(color);
	}

	void setFace3() {
		int face = 3;
		int pos = 0;
		int color = dino.cubeLoc[face][pos].face;
		if (DEBUG)
			System.out.println("face " + face + ": zeroethEdge " + color);
		setFace3Pos1(color);
		setFace3Pos2(color);
		setFace3Pos3(color);
	}

	void setFace4() {
		int face = 4;
		int pos = 0;
		int color = dino.cubeLoc[face][pos].face;
		if (DEBUG)
			System.out.println("face " + face + ": zeroethEdge " + color);
		/*
			findFacets(color);
			printPosition();
		*/
		setFace4Pos1(color);
		setFace4Pos2(color);
		setFace4Pos3(color);
	}

	int oppFace(int face) {
		switch (face) {
		case 0: return 4;
		case 1: return 3;
		case 2: return 5;
		case 3: return 1;
		case 4: return 0;
		case 5: return 2;
		default: return -1;
		}
	}

	int findAMixup(int face) {
		int i;

		for (i = face + 1; i < Dino.MAX_FACES; i++) {
			if ((dino.cubeLoc[i][3].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4) &&
					(dino.cubeLoc[i][1].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4)) {
				/* best */
				return i;
			}
		}
		for (i = face + 1; i < Dino.MAX_FACES; i++) {
			if ((dino.cubeLoc[i][3].rotation !=
					dino.cubeLoc[i][0].rotation) ||
					(dino.cubeLoc[i][1].rotation !=
					dino.cubeLoc[i][0].rotation)) {
				/* 2nd best */
				return i;
			}
		}
		return -1;
	}

	int startOfBadOrder(int face) {
		int i;
		for (i = 0; i < Dino.MAX_ORIENT; i++) {
			if ((dino.cubeLoc[face][i].rotation ==
					dino.cubeLoc[face][(i + 1) % Dino.MAX_ORIENT].rotation) &&
					(dino.cubeLoc[face][i].rotation !=
					dino.cubeLoc[face][(i + 2) % Dino.MAX_ORIENT].rotation))
				return (i + 2) % Dino.MAX_ORIENT;
			if ((dino.cubeLoc[face][i].rotation ==
					dino.cubeLoc[face][(i + 1) % Dino.MAX_ORIENT].rotation) &&
					(dino.cubeLoc[face][i].rotation ==
					dino.cubeLoc[face][(i + 2) % Dino.MAX_ORIENT].rotation))
				return -1; /* OK */
		}
		return -2; /* inverted */
	}

	void setFaceOrient() {
		int i, j, k;

		for (i = 0; i < Dino.MAX_FACES; i++) {
			j = startOfBadOrder(0);
			if (j >= 0) {
			 	oppSwap(0, j, Dino.CW, false);
				if ((j % 2) == 1)
					rotatePairs(0, 1);
				else
					rotatePairs(0, 3);
			 	oppSwap(0, j, Dino.CCW, false);
				if ((j % 2) == 1)
					rotatePairs(0, 1);
				else
					rotatePairs(0, 3);
			 	oppSwap(0, j, Dino.CW, false);
			}
			if (i == 3)
				movePuzzleFace(1, 0, Dino.BOTTOM, true);
			else if (i < 3)
				movePuzzleFace(0, 0, Dino.BOTTOM, true);
			else if (i == 4) {
				movePuzzleFace(0, 0, Dino.BOTTOM, true);
				movePuzzleFace(0, 0, Dino.BOTTOM, true);
			}
		
		}
		for (i = 0; i < Dino.MAX_FACES; i++) {
			j = oppFace(i);
			if ((dino.cubeLoc[i][3].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4) &&
					(dino.cubeLoc[i][1].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4) &&
					((dino.cubeLoc[j][3].rotation !=
					dino.cubeLoc[j][0].rotation) ||
					(dino.cubeLoc[j][1].rotation !=
					dino.cubeLoc[j][0].rotation))) {
				flipLR(i);
			}
		}
		for (i = 0; i < Dino.MAX_FACES; i++) {
			j = oppFace(i);
			if ((dino.cubeLoc[i][3].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4) &&
					(dino.cubeLoc[i][1].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4) &&
					(dino.cubeLoc[j][1].rotation ==
					dino.cubeLoc[j][0].rotation) &&
					(dino.cubeLoc[j][3].rotation ==
					dino.cubeLoc[j][0].rotation)) {
				k = findAMixup(i);
				if (k == -1)
					break;
				swapAdjFaces(i, k);
			}
		}
		for (i = 0; i < Dino.MAX_FACES; i++) {
			j = oppFace(i);
			if ((dino.cubeLoc[i][3].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4) &&
					(dino.cubeLoc[i][1].rotation ==
					(dino.cubeLoc[i][0].rotation + 2) % 4) &&
					((dino.cubeLoc[j][3].rotation !=
					dino.cubeLoc[j][0].rotation) ||
					(dino.cubeLoc[j][1].rotation !=
					dino.cubeLoc[j][0].rotation))) {
				flipLR(i);
			}
		}
		/* could have one inverted */
		if (startOfBadOrder(4) == -2)
			movePuzzleFace(0, 0, Dino.TOP, true);
		if (startOfBadOrder(1) == -2)
			movePuzzleFace(1, 0, Dino.TOP, true);
		else if (startOfBadOrder(2) == -2)
			movePuzzleFace(0, 0, Dino.TOP, true);
		else if (startOfBadOrder(3) == -2)
			movePuzzleFace(3, 0, Dino.TOP, true);
		else if (startOfBadOrder(5) == -2)
			movePuzzleFace(0, 0, Dino.BOTTOM, true);
		if (startOfBadOrder(0) == -2) {
		 	oppSwap(4, 0, Dino.CW, false);
			rotatePairs(0, 1);
		 	oppSwap(4, 0, Dino.CW, false);
			rotatePairs(0, 1);
		 	oppSwap(4, 0, Dino.CW, false);
			rotatePairs(0, 1);
		 	oppSwap(4, 0, Dino.CW, false);
			rotatePairs(0, 1);
		 	oppSwap(4, 0, Dino.CW, false);
		}
	}

	void solveFaces() {
		setFace0();
		setFace1();
		setFace2();
		setFace3();
		setFace4();
		if (dino.orientChoice) {
			setFaceOrient();
		}
	/*	int face = 4, pos = 0;
		for (face = 4; face < 5; face++) {
			for (pos = 0; pos < 4; pos++) {
				oppSwap(face, pos, Dino.CW, false);
				System.out.println("face " + face + "  pos " + pos);
				sleep(8);
				clearPieces();
			}
		}
	*/
	}

	/* This procedure coordinates the solution process. */
	void solvePieces() {
		if (!dino.checkSolved()) {
			if (dino.period == DinoInterface.PERIOD3)
				solveEdges();
			else if (dino.period == DinoInterface.PERIOD2)
				solveFaces();
			else {
				solveEdges();
				solveFaces();
			}
		}
		((DinoFrame) frame).callback(
			DinoInterface.ACTION_COMPUTED, dino.dim);
	}

	private class SeparateSubTask extends Thread {
		private boolean runFlag = false;
		SeparateSubTask() {
		}
		public void init() {
			start();
		}
		public void setFlag(boolean flag) {
			runFlag = flag;
		}
		public boolean getFlag() {
			return runFlag;
		}

		@Override
		public void run() {
			while (true) {
				try {
					sleep(100);
				} catch (InterruptedException e) {
					System.err.println("Interrupted");
				}
				if (runFlag) {
					solvePieces();
					runFlag = false;
				}
			}
		}
	}

	private SeparateSubTask sp = null;
	public void setFlag(boolean flag) {
		if (sp != null)
			sp.setFlag(flag);
	}
	public boolean getFlag() {
		if (sp == null)
			return false;
		return sp.getFlag();
	}
	public void init() {
		if (sp == null) {
			sp = new SeparateSubTask();
			sp.init();
		}
	}
}
